from gi.repository import Gtk, Gio, Gdk
from decimal import *

class Calc():

    def __init__(self, data):
        self.data = data

    def sumCategoryData(self,data_arr, category):
        total = 0
        for i in range (0,len(data_arr)):
            if data_arr[i][self.data.CATEGORY][self.data.CATEGORY_INDEX] == category:
                total += Decimal(data_arr[i][self.TRANSACTION_VALUE_INDEX])
        return total
    
    def sumCategoryMonthData(self,data_arr, category, month):
        total = 0
        for i in range (0,len(data_arr)):
            if data_arr[i][self.data.CATEGORY][self.data.CATEGORY_INDEX] == category and data_arr[i][self.data.DATE][self.data.DATE_MONTH] == month:
                total += Decimal(data_arr[i][self.TRANSACTION_VALUE_INDEX])
        return total
    
    def sumMonthData(self,data_arr, month):
        total = 0
        for i in range (0,len(data_arr)):
            if data_arr[i][self.data.DATE][self.data.DATE_MONTH] == month:
                total += Decimal(data_arr[i][self.TRANSACTION_VALUE_INDEX])
        return total
    
    def sumTotalData(self,data_arr):
        total = 0
        for i in range (0,len(data_arr)):
            total += Decimal(data_arr[i][self.data.TRANSACTION_VALUE_INDEX])
            print(total)
            #total = "%.2f" % total
        return total



#!/bin/python3

import os.path
import sys

# Check if we are working in the source tree or from the installed
# package and mangle the python path accordingly
realPath = os.path.realpath(sys.argv[0])  # If this file is run from a symlink, it needs to follow the symlink
if os.path.dirname(realPath) != ".":
    if realPath[0] == "/":
        fullPath = os.path.dirname(realPath)
    else:
        fullPath = os.getcwd() + "/" + os.path.dirname(realPath)
else:
    fullPath = os.getcwd()
sys.path.insert(0, os.path.dirname(fullPath))


from gi.repository import Gtk, Gio, Gdk
from budget.window import Window
from budget.data import Data

class main():

    def __init__(self):
        self.data = Data()
        self.data.import_data()
        self.win = Window(self.data)
        self.win.connect("delete-event", Gtk.main_quit)
        self.win.show_all()
    
        self.win.transactions.month_year_visible(True)
        self.win.transactions.range_visible(False)

        for i in range(0, len(self.win.transactions.menuListBox)):
            if self.win.transactions.editable_category(i):
                self.win.transactions.category_view_mode(i)
        
        self.win.connect('key-press-event', self.on_key_function)
        self.win.set_icon_name("budget.png")
        Gtk.main()

    def on_key_function(self, widget, event):
        # If ctrl is pressed
        if Gdk.ModifierType.CONTROL_MASK & event.state:
            # ctrl+q to quit
            if event.keyval == 113:
                Gtk.main_quit()
        # If ctrl is pressed
        if Gdk.ModifierType.MOD1_MASK & event.state:
            # alt+1
            if event.keyval == 49:
                self.win.notebook.set_current_page(0)
            # alt+2
            if event.keyval == 50:
                self.win.notebook.set_current_page(1)
            # alt+3
            #if event.keyval == 51:
            #    self.win.notebook.set_current_page(2)

if __name__=='__main__':
    main()



from gi.repository import Gtk, Gio
from decimal import *
import os.path, sys
import sqlite3 as lite

class Data():
    # Database location
    home = os.path.expanduser("~")
    local_path = home + '/.local/share/budget'
    db_path = local_path + '/budget.db'
    # For a separate development database, comment out the above 3 lines and uncomment the 1 line below
    #db_path = 'budget.db'
    
    # Create Database if none exists
    if(os.path.isfile(db_path) == False):
        if os.path.exists(local_path) == False:
            os.mkdir(local_path) 
        
        try:
            con = lite.connect(db_path)

            # Create Database Tables
            cur = con.cursor()
            cur.execute('create table categories(type INT, name VARCHAR(50), categoryOrder INT, categoryID INT PRIMARY KEY);')
            cur.execute('create table transactions(categoryID INT, year INT, month INT, day INT, value REAL, description VARCHAR(100), transactionID INT PRIMARY KEY);')
            cur.execute('create table projections(title VARCHAR(30), value REAL, description VARCHAR(50), categoryID INT, start_year INT, start_month INT, start_day INT, end_year INT, end_month INT, end_day INT, frequencyID INT, projectionID INTEGER PRIMARY KEY);')
            cur.execute('create table frequency(type VARCHAR(30), frequencyID INT PRIMARY KEY);')
            cur.execute('create table categoryType(type VARCHAR(30), typeID INT PRIMARY KEY);')

            # Initialize with data
            row = [("income", "0"),("expense","1")]
            cur.execute('INSERT INTO categoryType VALUES(?, ?)', row[0])
            cur.execute('INSERT INTO categoryType VALUES(?, ?)', row[1])
            
            row = [("0", "Uncategorized","-1","-1"),("1","Uncategorized","-2","-2")]
            cur.execute('INSERT INTO categories VALUES(?, ?, ?, ?)', row[0])
            cur.execute('INSERT INTO categories VALUES(?, ?, ?, ?)', row[1])
            
            row = [("One Time", "0"),("Daily","1"),("Weekly","2"),("Bi-Weekly","3"),("Monthly on Date", "4"),("Monthly on Weekday","5"),("Yearly","6")]
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[0])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[1])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[2])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[3])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[4])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[5])
            cur.execute('INSERT INTO frequency VALUES(?,?)', row[6])
            
            con.commit()

            data = cur.fetchone()

        except (lite.Error, e): 
            print("Error %s:" % e.args[0])
            sys.exit(1)

        finally:
            if con:
               con.close()

    # Menu Indexes
    MENU_TYPE_INDEX = 0
    MENU_NAME_INDEX = 1
    MENU_ORDER_INDEX = 2
    MENU_ID_INDEX = 3

    # Transactions Indexes
    TRANSACTION_MENU_INDEX = 0
    TRANSACTION_MENU_ID_INDEX = 0
    TRANSACTION_MENU_NAME_INDEX = 1

    TRANSACTION_DATE_INDEX = 1
    TRANSACTION_DATE_YEAR_INDEX = 0
    TRANSACTION_DATE_MONTH_INDEX = 1
    TRANSACTION_DATE_DAY_INDEX = 2

    TRANSACTION_VALUE_INDEX = 2
    TRANSACTION_DESCRIPTION_INDEX = 3
    TRANSACTION_ID_INDEX = 4

    INCOME_ORDER_ID = 0
    EXPENSE_ORDER_ID = 0
                
    PROJECTIONS_TITLE = 0
    PROJECTIONS_VALUE = 1
    PROJECTIONS_DESCRIPTION = 2
    PROJECTIONS_CATEGORY_ID = 3
    PROJECTIONS_CATEGORY_NAME = 4
    PROJECTIONS_CATEGORY_TYPE = 5
    PROJECTIONS_START_YEAR = 6
    PROJECTIONS_START_MONTH = 7
    PROJECTIONS_START_DAY = 8
    PROJECTIONS_END_YEAR = 9
    PROJECTIONS_END_MONTH = 10
    PROJECTIONS_END_DAY = 11
    PROJECTIONS_FREQUENCY = 12
    PROJECTIONS_ID = 13
    
    LATEST_ID = 0
    LATEST_MENU_ID = 0
    LATEST_PROJECTION_ID = 0

    def __init__(self):
        #Indexes for data arrays

        self.valid_numbers = ["1","2","3","4","5","6","7","8","9","0","."]
        self.transactionsMenu = []
        self.transactions = []
        self.incomeMenu = []
        self.expenseMenu = []
        self.yearMenu = []
        self.projections = []
        self.frequencyMenu = []

        self.transaction_view = 0
        self.overview = 0
        self.projections_view = 0
        
        self.allMonthMenu = [[0, "All"],
                            [1, "January"],
                            [2, "February"],
                            [3, "March"],
                            [4, "April"],
                            [5, "May"],
                            [6, "June"],
                            [7, "July"],
                            [8, "August"],
                            [9, "September"],
                            [10, "October"],
                            [11, "November"],
                            [12, "December"],
                            ]

    def add_category(self, menuType, category):
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            cur = con.cursor()
            
            if menuType == "income":
                cur.execute('SELECT MAX(categoryOrder) from categories where type = ' + str(0))
                row = cur.fetchone()
                self.INCOME_ORDER_ID = row[0]
                self.INCOME_ORDER_ID += 1
            if menuType == "expense":
                cur.execute('SELECT MAX(categoryOrder) from categories where type = ' + str(1))
                row = cur.fetchone()
                self.EXPENSE_ORDER_ID = row[0]
                self.EXPENSE_ORDER_ID += 1
            cur.execute('SELECT MAX(categoryID) from categories;')
            row = cur.fetchone()
            self.LATEST_MENU_ID = row[0]
            self.LATEST_MENU_ID += 1
            if(os.path.isfile('budget.db')):
                con = lite.connect(self.db_path)
                cur = con.cursor()
                
                typeID = []
                typeID.append(menuType)
                cur.execute("SELECT typeID from categoryType where type = ?", typeID)
                typeID = cur.fetchall()
                typeID = typeID[0][0]
                
                if menuType == "income":
                    row = [(str(typeID),str(category),str(self.INCOME_ORDER_ID),str(self.LATEST_MENU_ID))]
                if menuType == "expense":
                    row = [(str(typeID),str(category),str(self.EXPENSE_ORDER_ID),str(self.LATEST_MENU_ID))]
            
                cur = con.cursor()
                cur.execute('INSERT INTO categories VALUES(?,?,?,?)', row[0])
                con.commit()

                self.refresh_data()   
 
    def add_transaction(self, category, year, month, day, value, description, transactionID):
        for i in range(0,len(self.transactionsMenu)):
            if category == self.transactionsMenu[i][self.MENU_NAME_INDEX]:
                category = self.transactionsMenu[i][self.MENU_ID_INDEX]

        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            row = [(str(category),str(year),str(month),str(day),str(value),str(description),str(transactionID))]
        
            cur = con.cursor()
            cur.execute('INSERT INTO transactions VALUES(?,?,?,?,?,?,?)', row[0])
            con.commit()

            self.refresh_data()
    
    def add_projection(self, title, value, description, selected, category, startYear, startMonth, startDay, endYear, endMonth, endDay, frequency, projectionID):
        for i in range(0,len(self.transactionsMenu)):
            if category == self.transactionsMenu[i][self.MENU_NAME_INDEX]:
                category = self.transactionsMenu[i][self.MENU_ID_INDEX]

        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            cur = con.cursor()

            if selected == "income":
                selected = 0
            elif selected == "expense":
                selected = 1
             
            self.select = [(str(selected),str(category + 1))] 
            cur.execute("SELECT categoryID FROM categories WHERE categories.type = ? AND categories.categoryOrder = ?", self.select[0])
            self.categoryID = cur.fetchall()
            self.categoryID = self.categoryID[0][0]
            row = [(str(title),str(value),str(description),str(self.categoryID),str(startYear),str(startMonth),str(startDay),str(endYear),str(endMonth),str(endDay),str(frequency),str(projectionID))]
            cur.execute('INSERT INTO projections VALUES(?,?,?,?,?,?,?,?,?,?,?,?)', row[0])
            con.commit()
            
            self.refresh_data()
    
    def check_amount_value(self, widget, value, length, *args):
        if value not in self.valid_numbers:
            widget.stop_emission("insert-text")
            #print(value)
        # if boolean == False:
    

    def connect_data_views(self, transaction_view, overview, projections):
        self.transaction_view = transaction_view
        self.overview = overview
        self.projections_view = projections
        
    def delete_category(self, uniqueID):
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
        
            cur = con.cursor()
            cur.execute('SELECT type FROM categories WHERE categoryID = '+ str(uniqueID))
            row = cur.fetchone()
            if row[0] == 0:
                cur.execute('UPDATE transactions SET categoryID = -1 WHERE categoryID = ' + str(uniqueID))
                cur.execute('UPDATE projections SET categoryID = -1 where categoryID = ' + str(uniqueID));
            elif row[0] == 1:
                cur.execute('UPDATE transactions SET categoryID = -2 WHERE categoryID = ' + str(uniqueID))
                cur.execute('UPDATE projections SET categoryID = -2 where categoryID = ' + str(uniqueID));
            
            cur.execute('SELECT categoryOrder from categories where categoryID = ' + str(uniqueID))
            row = cur.fetchone()
            self.order = row[0]
            cur.execute('delete from categories where categoryID = ' + str(uniqueID))
            cur.execute('SELECT count(*) from categories;')
            row = cur.fetchone()
            self.count = row[0]
            for i in range(self.order, self.count):
                row = [(str(i), str(i + 1))]
                cur.execute('UPDATE categories set categoryOrder = ? where categoryOrder = ?', row[0])

            con.commit()
            
            self.refresh_data()   
    
    def delete_transaction(self, uniqueID):
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            cur = con.cursor()
            cur.execute('DELETE FROM transactions WHERE transactionID = ' + str(uniqueID))
            con.commit()

            self.refresh_data()
    
    def delete_projection(self, uniqueID):
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            cur = con.cursor()
            cur.execute('DELETE FROM projections WHERE projectionID = ' + str(uniqueID))
            con.commit()

            self.refresh_data()
    
    def edit_category(self, uniqueID, newLabel):
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            row = [(str(newLabel),str(uniqueID))]
        
            cur = con.cursor()
            cur.execute('UPDATE categories SET name = ? WHERE categoryID = ?', row[0])
            con.commit()

            self.refresh_data()
    
    def refresh_data(self):
            self.transactionsMenu = []
            self.transactions = []
            self.incomeMenu = []
            self.expenseMenu = []
            self.yearMenu = []
            self.projections = []
            self.frequencyMenu = []
            
            self.import_data()
            
            self.transaction_view.generate_sidebars()
            self.transaction_view.display_content()
            self.overview.redisplay_info()
            self.projections_view.redisplay_info()

    def import_data(self):
        
        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)

            cur = con.cursor()
            cur.execute('SELECT * FROM categories WHERE type = 0 ORDER BY categoryOrder;')
            rows = cur.fetchall()
            for row in rows:
                self.arr = []
                self.arr.append("income")                   # Type
                self.arr.append(row[1].strip())             # Name
                self.arr.append(row[2])                     # Order
                self.arr.append(row[3])                     # categoryID
            
                if self.INCOME_ORDER_ID < row[2]:
                    self.INCOME_ORDER_ID = row[2]
           
                if self.LATEST_MENU_ID < row[3]:
                    self.LATEST_MENU_ID = row[3]

                self.transactionsMenu.append(self.arr)
                
            cur = con.cursor()
            cur.execute('SELECT * FROM categories WHERE type = 1 ORDER BY categoryOrder;')
            rows = cur.fetchall()
            for row in rows:
                self.arr = []
                self.arr.append("expense")                  # Type
                self.arr.append(row[1].strip())             # Name
                self.arr.append(row[2])                     # Order
                self.arr.append(row[3])                     # categoryID
            
                if self.EXPENSE_ORDER_ID < row[2]:
                    self.EXPENSE_ORDER_ID = row[2]
           
                if self.LATEST_MENU_ID < row[3]:
                    self.LATEST_MENU_ID = row[3]
                
                self.transactionsMenu.append(self.arr)
                
            for i in range(0, len(self.transactionsMenu)):
                if self.transactionsMenu[i][self.MENU_TYPE_INDEX] == "income":
                    self.incomeMenu.append(self.transactionsMenu[i][self.MENU_ID_INDEX])
                if self.transactionsMenu[i][self.MENU_TYPE_INDEX] == "expense":
                    self.expenseMenu.append(self.transactionsMenu[i][self.MENU_ID_INDEX])
               
                 
            cur.execute('SELECT * FROM transactions;')    
            rows = cur.fetchall()
            for row in rows:
                self.arr = []
                self.catArr = []
                self.dateArr = []
                self.catArr.append(row[0])                  # categoryID
                for i in range(0, len(self.transactionsMenu)):
                    if row[0] == self.transactionsMenu[i][3]:
                        self.catArr.append(self.transactionsMenu[i][1])     # Name
                self.dateArr.append(row[1])                 # year
                self.dateArr.append(row[2])                 # month
                self.dateArr.append(row[3])                 # day
                self.arr.append(self.catArr)
                self.arr.append(self.dateArr)
                self.arr.append(row[4])                     # Value
                self.arr.append(row[5].strip())             # Description
                self.arr.append(row[6])                     # transactionID
                if self.LATEST_ID < row[6]:
                    self.LATEST_ID = row[6]
                self.sort_transaction(self.transactions, self.arr)
            
            cur = con.cursor()
            cur_frequency = con.cursor()
            cur_categoryNameType = con.cursor()

            cur.execute('SELECT * FROM projections;')
            rows = cur.fetchall()
            for row in rows:
                cur_frequency.execute('SELECT frequency.type from frequency, projections where frequency.frequencyID = projections.frequencyID and projections.projectionID = ?;', (str(row[11]),))
                self.frequency = cur_frequency.fetchall()
                self.frequency = self.frequency[0][0]
                
                cur_categoryNameType.execute("SELECT name, type FROM categories WHERE categoryID = ?", (str(row[3]),))

                self.categoryNameType = cur_categoryNameType.fetchall()
                self.categoryName = self.categoryNameType[0][0]
                self.categoryType = self.categoryNameType[0][1]
                
                self.arr = []
                self.arr.append(row[0].strip())             # Title
                self.arr.append(row[1])                     # Value
                self.arr.append(row[2].strip())             # Description
                self.arr.append(row[3])                     # Category ID
                self.arr.append(self.categoryName)          # Category Name
                self.arr.append(self.categoryType)          # Category Type
                self.arr.append(row[4])                     # Start Year
                self.arr.append(row[5])                     # Start Month
                self.arr.append(row[6])                     # Start Day
                self.arr.append(row[7])                     # End Year
                self.arr.append(row[8])                     # End Month
                self.arr.append(row[9])                     # End Day
                self.arr.append(self.frequency)             # Frequency ID
                self.arr.append(row[11])                    # Projection ID
                
                if self.LATEST_PROJECTION_ID < row[11]:
                    self.LATEST_PROJECTION_ID = row[11]
            
                self.projections.append(self.arr)
        
            cur = con.cursor()
            cur.execute('SELECT DISTINCT year FROM transactions ORDER BY year DESC;') 
            rows = cur.fetchall()
            
            self.yearMenu.append("All")
            for row in rows:
                self.yearMenu.append(str(row[0]))
            
            cur = con.cursor()
            cur.execute('SELECT *  FROM frequency;') 
            rows = cur.fetchall()
            for row in rows:
                self.arr = []
                self.arr.append(row[0].strip())             # Title
                self.arr.append(row[1])                     # Frequency ID
                self.frequencyMenu.append(self.arr)

    def sort_transaction(self, data, arr):
        if len(data) == 0:
            data.append(arr)
        else:
            flag = False
            for i in range(len(data)):
                # If entry's year is equal to array's year
                if data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_YEAR_INDEX] == int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_YEAR_INDEX]):
                    # If entry's month is equal to array's month
                    if data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX]):
                        # If entry's day is equal to array's day
                        if data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX]):
                            data.insert(i, arr)
                            flag = True
                            break
                        # If entry's day is less than array's day
                        elif data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] > int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX]):
                            for j in range(i, len(data)):
                                if data[j][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX]):
                                    if data[j][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] <= int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX]):
                                        data.insert(j, arr)
                                        flag = True
                                        break
                                else:
                                    data.insert(j , arr)
                                    flag = True
                                    break
                            break
                        # If entry's day is greater than array's day
                        elif data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] < int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX]):
                            data.insert(i , arr)
                            flag = True
                            break
                    # If entry's month is less than array's month
                    elif data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] < int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX]):
                        data.insert(i , arr)
                        flag = True
                        break
                # If entry's year is less than income array's year
                elif data[i][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_YEAR_INDEX] < int(arr[self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_YEAR_INDEX]):
                    data.insert(i , arr)
                    flag = True
                    break

            if flag == False:
                data.append(arr)
    
    def sort_projections(self, data, arr):
        if len(data) == 0:
            data.append(arr)
        else:
            flag = False
            for i in range(len(data)):
                # If entry's year is equal to array's year
                if data[i][self.PROJECTIONS_START_YEAR] == int(arr[self.PROJECTIONS_START_YEAR]):
                    # If entry's month is equal to array's month
                    if data[i][self.PROJECTIONS_START_MONTH] == int(arr[self.PROJECTIONS_START_MONTH]):
                        # If entry's day is equal to array's day
                        if data[i][self.PROJECTIONS_START_DAY] == int(arr[self.PROJECTIONS_START_DAY]):
                            data.insert(i, arr)
                            flag = True
                            break
                        # If entry's day is less than array's day
                        elif data[i][self.PROJECTIONS_START_DAY] > int(arr[self.PROJECTIONS_START_DAY]):
                            for j in range(i, len(data)):
                                if data[j][self.PROJECTIONS_START_MONTH] == int(arr[self.PROJECTIONS_START_MONTH]):
                                    if data[j][self.PROJECTIONS_START_DAY] <= int(arr[self.PROJECTIONS_START_DAY]):
                                        data.insert(j, arr)
                                        flag = True
                                        break
                                else:
                                    data.insert(j , arr)
                                    flag = True
                                    break
                            break
                        # If entry's day is greater than array's day
                        elif data[i][self.PROJECTIONS_START_DAY] < int(arr[self.PROJECTIONS_START_DAY]):
                            data.insert(i , arr)
                            flag = True
                            break
                    # If entry's month is less than array's month
                    elif data[i][self.PROJECTIONS_START_MONTH] < int(arr[self.PROJECTIONS_START_MONTH]):
                        data.insert(i , arr)
                        flag = True
                        break
                # If entry's year is less than income array's year
                elif data[i][self.PROJECTIONS_START_YEAR] < int(arr[self.PROJECTIONS_START_YEAR]):
                    data.insert(i , arr)
                    flag = True
                    break

            if flag == False:
                data.append(arr)

        return data
           
    def translate_date(self,data,index):
        dateString = ""
        
        if index == "edit":
            if data[1] == 0:
                dateString += ("January")
            elif data[1] == 1:
                dateString += ("February")
            elif data[1] == 2:
                dateString += ("March")
            elif data[1] == 3:
                dateString += ("April")
            elif data[1] == 4:
                dateString += ("May")
            elif data[1] == 5:
                dateString += ("June")
            elif data[1] == 6:
                dateString += ("July")
            elif data[1] == 7:
                dateString += ("August")
            elif data[1] == 8:
                dateString += ("September")
            elif data[1] == 9:
                dateString += ("October")
            elif data[1] == 10:
                dateString += ("November")
            elif data[1] == 11:
                dateString += ("December")
            else:
                dateString += ("Month Fail")
            
            dateString += (" " + str(data[2]))

            if data[2] == 1:
                dateString += ("st")
            elif data[2] == 21:
                dateString += ("st")
            elif data[2] == 31:
                dateString += ("st")
            elif data[2] == 2:
                dateString += ("nd")
            elif data[2] == 22:
                dateString += ("nd")
            elif data[2] == 3:
                dateString += ("rd")
            elif data[2] == 23:
                dateString += ("rd")
            else:
                dateString += ("th")
            
            dateString += ", "
            dateString += str(data[0])
        
        elif index == "month":
            if data == "January":
                dateString = 1
            elif data == "February":
                dateString = 2
            elif data == "March":
                dateString = 3
            elif data == "April":
                dateString = 4
            elif data == "May":
                dateString = 5
            elif data == "June":
                dateString = 6
            elif data == "July":
                dateString = 7
            elif data == "August":
                dateString = 8
            elif data == "September":
                dateString = 9
            elif data == "October":
                dateString = 10
            elif data == "November":
                dateString = 11
            elif data == "December":
                dateString  = 12
            else:
                dateString = ("Month Fail")
            
        else:
            if data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 1:
                dateString += ("January")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 2:
                dateString += ("February")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 3:
                dateString += ("March")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 4:
                dateString += ("April")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 5:
                dateString += ("May")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 6:
                dateString += ("June")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 7:
                dateString += ("July")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 8:
                dateString += ("August")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 9:
                dateString += ("September")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 10:
                dateString += ("October")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 11:
                dateString += ("November")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_MONTH_INDEX] == 12:
                dateString += ("December")
            else:
                dateString += ("Month Fail")

            dateString += (" " + str(data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX]))

            if data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 1:
                dateString += ("st")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 21:
                dateString += ("st")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 31:
                dateString += ("st")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 2:
                dateString += ("nd")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 22:
                dateString += ("nd")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 3:
                dateString += ("rd")
            elif data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_DAY_INDEX] == 23:
                dateString += ("rd")
            else:
                dateString += ("th")

            dateString += ", "
            dateString += str(data[index][self.TRANSACTION_DATE_INDEX][self.TRANSACTION_DATE_YEAR_INDEX])
                
        return dateString
    
    def update_transaction(self, category, year, month, day, value, description, transactionID):
        for i in range(0,len(self.transactionsMenu)):
            if category == self.transactionsMenu[i][self.MENU_NAME_INDEX]:
                category = self.transactionsMenu[i][self.MENU_ID_INDEX]

        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            row = [(str(category),str(year),str(month),str(day),str(value),str(description),str(transactionID))]

            cur = con.cursor()
            cur.execute('update transactions set categoryID = ?, year = ?, month = ?, day = ?, value = ?, description = ? where transactionID = ?', row[0])
            con.commit()

            self.refresh_data()            
    
    def update_projection(self, title, category, year, month, day, value, description, projectionID):
        for i in range(0,len(self.transactionsMenu)):
            if category == self.transactionsMenu[i][self.MENU_NAME_INDEX]:
                category = self.transactionsMenu[i][self.MENU_ID_INDEX]

        if(os.path.isfile(self.db_path)):
            con = lite.connect(self.db_path)
            row = [(str(title),str(category),str(year),str(month),str(day),str(value),str(description),str(projectionID))]

            cur = con.cursor()
            cur.execute('update projections set title=?, categoryID = ?, start_year = ?, start_month = ?, start_day = ?, value = ?, description = ? where projectionID = ?', row[0])
            con.commit()
            
            self.refresh_data()            





from gi.repository import Gtk, Gio, Gdk

class Add_Entry_Popover(Gtk.Window):

    def __init__(self, data):
        #Initialize Data
        self.data = data
        # Create Widgets
        self.addGrid = Gtk.Grid()
        self.addIncomeRadio = Gtk.RadioButton.new_with_label(None, "Income")
        self.addExpenseRadio = Gtk.RadioButton.new_with_label(None, "Expense")
        self.addExpenseRadio.join_group(self.addIncomeRadio)
        self.radioBox = Gtk.Box(Gtk.Orientation.HORIZONTAL,1)
        self.radioBox.pack_start(self.addIncomeRadio, True, True, 0)
        self.radioBox.pack_start(self.addExpenseRadio, True, True, 0)
        self.addCategoryLabel = Gtk.Label("Category")
        self.addCategoryComboBoxText = Gtk.ComboBoxText()
        self.addEntryBox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.addEntryLabel = Gtk.Label("Entry")
        self.addDescriptionLabel = Gtk.Label("Description")
        self.addCurrencyLabel = Gtk.Label("$ ")
        self.addEntry = Gtk.Entry()
        self.addEntry.set_input_purpose(Gtk.InputPurpose.NUMBER)
        self.addDescription = Gtk.Entry()
        self.addDate = Gtk.Calendar()
        self.addSubmitButton = Gtk.Button("Submit")
        
        self.addEntryBox.add(self.addCurrencyLabel)
        self.addEntryBox.add(self.addEntry)
        
        # Style Widgets
        self.radioBox.set_homogeneous(True)
        self.radioBox.set_hexpand(True)        
        self.radioBox.set_vexpand(True)        
        self.radioBox.set_property("height-request", 34)
        self.add_popover_margin(self.radioBox, 10)
        self.addIncomeRadio.set_property("draw-indicator",False)
        self.addExpenseRadio.set_property("draw-indicator",False)
        Gtk.StyleContext.add_class(self.radioBox.get_style_context(), "linked")
        
        self.add_popover_margin(self.addCategoryComboBoxText, 10)
        self.add_popover_margin(self.addEntryBox, 10)
        self.add_popover_margin(self.addDescription, 10)
        self.addCategoryComboBoxText.set_margin_top(0)
        self.addEntryBox.set_margin_top(0)
        self.addEntryBox.set_margin_end(4)
        self.addDescription.set_margin_top(0)
        self.addDescription.set_margin_start(4)
        self.add_popover_margin(self.addDate, 10)
        self.add_popover_margin(self.addSubmitButton, 10)

        self.addCategoryComboBoxText.set_property("height-request", 34)

        self.radioStatus = "income"
        
        # Connect Widget Handlers
        self.addEntry.connect("insert-text", self.data.check_amount_value)
        self.addIncomeRadio.connect("toggled", self.on_addRadio_toggled)
        
        # Add Widgets to Grid
        self.addGrid.attach(self.radioBox,0,0,2,1)
        
        self.addGrid.attach(self.addCategoryLabel,0,1,2,1)
        self.addGrid.attach(self.addCategoryComboBoxText,0,2,2,1)
        self.addGrid.attach(self.addEntryLabel,0,3,1,1)
        self.addGrid.attach(self.addDescriptionLabel,1,3,1,1)
        self.addGrid.attach(self.addEntryBox,0,4,1,1)
        self.addGrid.attach(self.addDescription,1,4,1,1)
        self.addGrid.attach(self.addDate,0,5,2,1)
        self.addGrid.attach(self.addSubmitButton,0,6,2,1)
        self.addEntry.connect("activate", self.on_addSubmitButton_clicked)
        self.addDescription.connect("activate", self.on_addSubmitButton_clicked)
        self.addSubmitButton.connect("clicked", self.on_addSubmitButton_clicked)
    
    def add_popover_margin(self, widget, margin):
        widget.set_margin_start(margin)
        widget.set_margin_top(margin)
        widget.set_margin_end(margin)
        widget.set_margin_bottom(margin)
    
    def on_addButton_clicked(self, button, addPopover):
        if addPopover.get_visible():
            addPopover.hide()
        else:
            for i in range(0, len(self.data.transactionsMenu)):
                self.addCategoryComboBoxText.remove(0)
            
            if self.addIncomeRadio.get_active() == True:
                self.selected = "income"
            elif self.addExpenseRadio.get_active() == True:
                self.selected = "expense"
            
            for i in range(0,len(self.data.transactionsMenu)):
                if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == self.selected:
                    if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                        self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
            addPopover.show_all()

    def on_addRadio_toggled(self, *args):
        for i in range(0, len(self.data.transactionsMenu)):
            self.addCategoryComboBoxText.remove(0)
        if self.addIncomeRadio.get_active() == True:
            self.radioStatus = "income"
            for i in range(0,len(self.data.transactionsMenu)):
                if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == "income":
                    if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                        self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
        elif self.addExpenseRadio.get_active() == True:
            self.radioStatus = "expense"
            for i in range(0,len(self.data.transactionsMenu)):
                if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == "expense":
                    if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                        self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
    
    def on_addSubmitButton_clicked(self, *args):
        
        if self.addCategoryComboBoxText.get_active() < 0:
            self.addCategoryLabel.set_markup("<span foreground=\"red\"><b>* Category</b></span>")
        else:
            self.addCategoryLabel.set_text("Category")
        
        if self.addEntry.get_text() == "":
            self.addEntryLabel.set_markup("<span foreground=\"red\"><b>* Entry</b></span>")
        else:
            self.addEntryLabel.set_text("Entry")
        
        if self.addCategoryComboBoxText.get_active() >= 0 and self.addEntry.get_text() != "":
            self.dateArr = self.addDate.get_date()
            self.year = str(self.dateArr[0])
            self.month = str(self.dateArr[1] + 1)
            self.day = str(self.dateArr[2])
            self.data.LATEST_ID += 1
           
            self.data.add_transaction(self.addCategoryComboBoxText.get_active_text(), self.year, self.month, self.day, 
                                self.addEntry.get_text(), self.addDescription.get_text(), self.data.LATEST_ID)
            
            #self.addEntry.set_text("")
            self.addDescription.set_text("")


class Add_Category_Popover(Gtk.Window):

    def __init__(self, data):
        #Initialize Data
        self.data = data
        # Create Widgets
        self.addGrid = Gtk.Grid()
        self.addIncomeRadio = Gtk.RadioButton.new_with_label(None, "Income")
        self.addExpenseRadio = Gtk.RadioButton.new_with_label(None, "Expense")
        self.addExpenseRadio.join_group(self.addIncomeRadio)
        self.radioBox = Gtk.Box(Gtk.Orientation.HORIZONTAL,1)
        self.radioBox.pack_start(self.addIncomeRadio, True, True, 0)
        self.radioBox.pack_start(self.addExpenseRadio, True, True, 0)
        self.addEntryLabel = Gtk.Label("Category")
        self.addEntry = Gtk.Entry()
        self.addCancelButton = Gtk.Button("Cancel")
        self.addSubmitButton = Gtk.Button("Submit")
        
        # Style Widgets
        self.radioBox.set_homogeneous(True)
        self.radioBox.set_hexpand(True)        
        self.radioBox.set_vexpand(True)        
        self.radioBox.set_property("height-request", 34)
        self.add_popover_margin(self.radioBox, 10)
        self.addIncomeRadio.set_property("draw-indicator",False)
        self.addExpenseRadio.set_property("draw-indicator",False)
        Gtk.StyleContext.add_class(self.radioBox.get_style_context(), "linked")
        self.add_popover_margin(self.addEntryLabel, 1)
        self.add_popover_margin(self.addEntry, 10)
        self.addEntry.set_margin_top(0)
        self.add_popover_margin(self.addCancelButton, 10)
        self.add_popover_margin(self.addSubmitButton, 10)

        self.radio = "income"
        
        # Connect Widget Handlers
        self.addIncomeRadio.connect("toggled", self.on_addRadio_toggled)
        
        # Add Widgets to Grid
        self.addGrid.attach(self.radioBox,0,0,2,1)
        
        self.addGrid.attach(self.addEntryLabel,0,1,2,1)
        self.addGrid.attach(self.addEntry,0,2,2,1)
        self.addGrid.attach(self.addCancelButton,0,3,1,1)
        self.addGrid.attach(self.addSubmitButton,1,3,1,1)
        self.addCancelButton.connect("clicked", self.on_addCancelButton_clicked)
        self.addSubmitButton.connect("clicked", self.on_addSubmitButton_clicked)
        self.addEntry.connect("activate", self.on_addSubmitButton_clicked)
    
    def add_popover_margin(self, widget, margin):
        widget.set_margin_start(margin)
        widget.set_margin_top(margin)
        widget.set_margin_end(margin)
        widget.set_margin_bottom(margin)
    
    def on_addCancelButton_clicked(self, button):
        self.addPopover.hide()
    
    def on_addCategoryButton_clicked(self, button, addPopover):
        self.addPopover = addPopover
        if addPopover.get_visible():
            self.addPopover.hide()
        else:
            self.addPopover.show_all()
            self.addEntry.set_text("")
            self.addEntry.grab_focus()
    
    def on_addRadio_toggled(self, *args):
        if self.addIncomeRadio.get_active() == True:
            self.radio = "income"
        elif self.addExpenseRadio.get_active() == True:
            self.radio = "expense"

    def on_addSubmitButton_clicked(self, button):
        self.entryString = ""
        
        if self.addEntry.get_text() == "":
            self.addEntry.set_placeholder_text("Enter A Category")
        else:
            self.data.add_category(self.radio, self.addEntry.get_text())

            # Refresh the menu
            self.data.transaction_view.generate_sidebars()
            
            self.addPopover.hide()



from gi.repository import Gtk, Gio, Gdk
from decimal import *
from budget.calc import Calc

class Overview():

    def __init__(self, data):
        
        self.data = data
        # Initialize Variables
        self.calc = Calc(self.data)
        self.monthArr = []
        self.categoryArr = []
        self.entryRows = []
        self.index = 0
        self.monthIndex = 10000
        self.categoryIndex = 10000
        
        self.menuColor = .75
        self.totalColor = .75
        self.highlightColor = .88
        
        # Create Layouts
        self.grid = Gtk.Grid(name="overviewGrid")
        self.overviewGrid = Gtk.Grid()
        self.headerGrid = Gtk.Grid()
        
        self.monthGrid = Gtk.Grid()
        self.monthScrolledWindow = Gtk.ScrolledWindow()
        self.monthViewport = Gtk.Viewport()
        
        self.contentGrid = Gtk.Grid()
        self.contentScrolledWindow = Gtk.ScrolledWindow()
        self.contentViewport = Gtk.Viewport()
        
        self.categoryGrid = Gtk.Grid()
        self.categoryScrolledWindow = Gtk.ScrolledWindow()
        self.categoryViewport = Gtk.Viewport()
        
        # Build Master Grid
        self.grid.attach(self.headerGrid,0,0,1,1)
        self.grid.attach(self.overviewGrid,0,1,1,1)
        
        # Style Master Grid
        
        self.build_header()
        self.build_overview()    
        self.display_info("income", self.data.transactions)
        self.empty_row()
        self.display_info("expense",  self.data.transactions)

    def build_header(self):
        # Build Header Grid
        self.blankLabel = Gtk.Label()
        
        self.balanceLabel = Gtk.Label("Balance:  ")
        self.balanceLabel.set_halign(Gtk.Align.END)
        #self.balanceTotalLabel = Gtk.Label( "$" + str(self.calc.sumTotalData(self.data.transactions) - self.calc.sumTotalData(self.data.transactions)))
        #self.balanceTotalLabel.set_halign(Gtk.Align.START)
        
        self.varianceLabel = Gtk.Label("Variance:  ")
        self.varianceLabel.set_halign(Gtk.Align.END)
        #self.varianceTotalLabel = Gtk.Label( "$" + str(self.calc.sumTotalData(self.data.transactions)))
        #self.varianceTotalLabel.set_halign(Gtk.Align.START)
        
        self.incomeTotalLabel = Gtk.Label("Total Income:  ")
        self.incomeTotalLabel.set_halign(Gtk.Align.END)
        #self.incomeTotalValueLabel = Gtk.Label( "$" + str(self.calc.sumTotalData(self.data.transactions)))
        #self.incomeTotalValueLabel.set_halign(Gtk.Align.START)
        
        self.expensesTotalLabel = Gtk.Label("Total Expenses:  ")
        self.expensesTotalLabel.set_halign(Gtk.Align.END)
        #self.expensesTotalValueLabel = Gtk.Label( "$" + str(self.calc.sumTotalData(self.data.transactions)))
        #self.expensesTotalValueLabel.set_halign(Gtk.Align.START)
        
        #self.headerGrid.attach(self.blankLabel,0,0,5,1)
        
        #self.headerGrid.attach(self.balanceLabel,0,1,1,1)
        #self.headerGrid.attach(self.balanceTotalLabel,1,1,1,1)
        
        #self.headerGrid.attach(self.varianceLabel,0,2,1,1)
        #self.headerGrid.attach(self.varianceTotalLabel,1,2,1,1)
        
        #self.headerGrid.attach(self.expensesTotalLabel,3,1,1,1)
        #self.headerGrid.attach(self.expensesTotalValueLabel,4,1,1,1)
        
        #self.headerGrid.attach(self.incomeTotalLabel,3,2,1,1)
        #self.headerGrid.attach(self.incomeTotalValueLabel,4,2,1,1)
        
        #for i in range(0,5):
        #    self.dummyHeaderLabel = Gtk.Label()
        #    self.headerGrid.attach(self.dummyHeaderLabel,i,4,1,1)
        
        # Style Header Grid
        #self.headerGrid.set_column_homogeneous(True)
        #self.headerGrid.set_hexpand(True)
       

    def build_overview(self):
        # Build Overview Grid
        self.clearButton = Gtk.Button("Clear Selection")
        self.clearButton.connect("clicked", self.clear_selection)
        self.overviewGrid.attach(self.clearButton,0,0,1,1)
        
        self.monthViewport.add(self.monthGrid)
        self.monthScrolledWindow.add(self.monthViewport)
        self.overviewGrid.attach(self.monthScrolledWindow,1,0,1,1)
        
        self.categoryViewport.add(self.categoryGrid)
        self.categoryScrolledWindow.add(self.categoryViewport)
        self.overviewGrid.attach(self.categoryScrolledWindow,0,1,1,1)
       
        self.contentViewport.add(self.contentGrid)
        self.contentScrolledWindow.add(self.contentViewport)
        self.overviewGrid.attach(self.contentScrolledWindow,1,1,1,1)
        
        # Print out Months
        for index in range(1,len(self.data.allMonthMenu) + 1):
            # Total Header
            if index == len(self.data.allMonthMenu):
                self.button = Gtk.Button("Total")
                self.button.set_relief(Gtk.ReliefStyle.NONE)
                self.button.set_property("height-request", 30)
                self.button.set_property("width-request", 120)
                self.monthGrid.attach(self.button,index,0,1,1)
                self.monthArr.append([index, self.button])
                self.button.connect("clicked", self.month_clicked, 13)
            else:
                self.button = Gtk.Button(self.data.allMonthMenu[index][1])
                self.button.set_relief(Gtk.ReliefStyle.NONE)
                self.button.set_property("height-request", 30)
                self.button.set_property("width-request", 120)
                self.monthGrid.attach(self.button,index,0,1,1)
                self.monthArr.append([index, self.button])
                self.button.connect("clicked", self.month_clicked, index)
        
        # Style Overview Grid
        self.clearButton.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.menuColor, self.menuColor, self.menuColor, self.menuColor))
        
        self.categoryGrid.set_column_homogeneous(True)
        self.categoryGrid.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.menuColor, self.menuColor, self.menuColor, self.menuColor))
        
        self.categoryScrolledWindow.set_vexpand(True)
        self.categoryScrolledWindow.set_property("width-request",150)
        self.categoryScrolledWindow.set_property("hscrollbar-policy",Gtk.PolicyType.NEVER)
        self.categoryScrolledWindow.set_vadjustment(self.contentScrolledWindow.get_vadjustment())
        
        self.categoryVScrollBar = self.categoryScrolledWindow.get_vscrollbar()
        self.categoryVScrollBar.set_property("visible",False)
        
        self.monthGrid.set_column_homogeneous(True)
        self.monthGrid.set_hexpand(True)
        self.monthGrid.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.menuColor, self.menuColor, self.menuColor, self.menuColor))
        self.monthScrolledWindow.set_property("vscrollbar-policy",Gtk.PolicyType.NEVER)
        self.monthScrolledWindow.set_hadjustment(self.contentScrolledWindow.get_hadjustment())
        
        self.monthHScrollBar = self.monthScrolledWindow.get_hscrollbar()
        self.monthHScrollBar.set_property("visible",False)
        
        self.contentScrolledWindow.set_vexpand(True)
        #self.contentGrid.set_column_homogeneous(True)
        self.contentGrid.set_hexpand(True)
    
    def category_clicked(self, button, index):
        self.categoryIndex = index
        for i in range (0, len(self.categoryArr)):
            if self.categoryArr[i][0] == index:
                self.categoryArr[i][1].set_relief(Gtk.ReliefStyle.HALF)
                for j in range(0, len(self.entryRows[i][1])):
                    if j == self.monthIndex:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.menuColor, self.menuColor, self.menuColor, self.menuColor))
                    else:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.highlightColor, self.highlightColor, self.highlightColor, self.highlightColor))
            else:
                self.categoryArr[i][1].set_relief(Gtk.ReliefStyle.NONE)
                for j in range(0, len(self.entryRows[i][1])):
                    if j != self.monthIndex:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(.0, .0, .0, .0));
                    else:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.highlightColor, self.highlightColor, self.highlightColor, self.highlightColor))
    
    def clear_selection(self, button):
        self.categoryIndex = 10000
        self.monthIndex = 10000
        for i in range (0, len(self.categoryArr)):
            self.categoryArr[i][1].set_relief(Gtk.ReliefStyle.NONE)
        for i in range (0, len(self.monthArr)):
            self.monthArr[i][1].set_relief(Gtk.ReliefStyle.NONE)
        for i in range(0, len(self.entryRows)):
            for j in range(0, len(self.entryRows[i][1])):
                # Get count of income categories
                count = 0
                for k in range(0, len(self.data.transactionsMenu)):
                    if self.data.transactionsMenu[k][self.data.MENU_TYPE_INDEX] == "income":
                        count += 1
                # Reset All Income
                if self.entryRows[i][0] == self.categoryArr[count][0]:
                    self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                # Reset All Expenses
                elif self.entryRows[i][0] == self.categoryArr[len(self.data.transactionsMenu) + 2][0]:
                    self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                else:
                    if j == len(self.data.allMonthMenu) - 1:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                    else:
                        self.entryRows[i][1][j].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(.0, .0, .0, .0));
    
    def display_info(self, category, data_arr):
        # Print out Categories
        for index in range(0,len(self.data.transactionsMenu)):
            if self.data.transactionsMenu[index][self.data.MENU_TYPE_INDEX] == category:
                self.button = Gtk.Button(self.data.transactionsMenu[index][self.data.MENU_NAME_INDEX])
                self.button.set_relief(Gtk.ReliefStyle.NONE)
                self.button.set_property("height-request", 40)
                self.contentArr = []
                self.categoryArr.append([self.index, self.button])
                self.button.connect("clicked", self.category_clicked, self.index)
                self.categoryGrid.attach(self.button, 0, self.index, 1, 1)
                
                # Print out total values for each category for each month
                for month in range(1,len(self.data.allMonthMenu) + 1):
                    self.total = 0
                    # Total Column
                    if month == len(self.data.allMonthMenu):
                        for data in range(0,len(data_arr)):
                            if data_arr[data][0][0] == index:
                                self.total += Decimal(data_arr[data][2])
                        self.totalLabel = Gtk.Label()
                        self.totalLabel.set_markup("<b>$" + str(self.total) + "</b>")
                        self.totalLabel.set_property("height-request", 40)
                        self.totalLabel.set_property("width-request", 120)
                        self.totalLabel.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                        self.contentArr.append(self.totalLabel)
                        self.contentGrid.attach(self.totalLabel, month - 1, self.index, 1, 1) 
                    else:
                        for data in range(0,len(data_arr)):
                            if data_arr[data][self.data.TRANSACTION_MENU_INDEX][self.data.TRANSACTION_MENU_NAME_INDEX] == index:
                                if data_arr[data][self.data.TRANSACTION_DATE_INDEX][self.data.TRANSACTION_DATE_MONTH] == self.data.allMonthMenu[month][0]:
                                    self.total += Decimal(data_arr[data][2])
                        self.totalLabel = Gtk.Label("$" + str(self.total))
                        self.totalLabel.set_property("height-request", 40)
                        self.totalLabel.set_property("width-request", 120)
                        self.contentArr.append(self.totalLabel)
                        self.contentGrid.attach(self.totalLabel, month - 1, self.index, 1, 1) 
                self.entryRows.append([self.index, self.contentArr])
                self.index += 1
        
        # Print out "All" Button
        if category == "income":
            self.AllButton = Gtk.Button("All Income")
        elif category == "expense":
            self.AllButton = Gtk.Button("All Expenses")
        self.AllButton.set_relief(Gtk.ReliefStyle.NONE)
        self.AllButton.set_property("height-request", 40)
        self.categoryArr.append([self.index, self.AllButton])
        self.AllButton.connect("clicked", self.category_clicked, self.index)
        self.categoryGrid.attach(self.AllButton, 0, self.index, 1, 1)

        # Print out total values of all categories for each month 
        self.contentArr = []
        for month in range(1,len(self.data.allMonthMenu) + 1):
            self.total = 0
            # Total Label
            if month == len(self.data.allMonthMenu):
                for data in range(0,len(data_arr)):
                    self.total += Decimal(data_arr[data][2])
                self.totalLabel = Gtk.Label()
                self.totalLabel.set_markup("<b>$" + str(self.total) + "</b>")
                self.totalLabel.set_property("height-request", 40)
                self.totalLabel.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                self.contentArr.append(self.totalLabel)
                self.contentGrid.attach(self.totalLabel, month - 1, self.index, 1, 1) 
            else:
                for data in range(0,len(data_arr)):
                    if data_arr[data][1][0] == self.data.allMonthMenu[month][0]:
                        self.total += Decimal(data_arr[data][2])
                self.totalLabel = Gtk.Label()
                self.totalLabel.set_markup("<b>$" + str(self.total) + "</b>")
                self.totalLabel.set_property("height-request", 40)
                self.totalLabel.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
                self.contentArr.append(self.totalLabel)
                self.contentGrid.attach(self.totalLabel, month - 1, self.index, 1, 1) 
        self.entryRows.append([self.index, self.contentArr])
        self.index += 1
       
    def empty_row(self):
        # Set up empty Row
        self.contentArr = []
        self.dummyCategoryButton = Gtk.Button()
        self.dummyCategoryButton.set_relief(Gtk.ReliefStyle.NONE)
        self.dummyCategoryButton.set_property("height-request", 30)
        self.dummyCategoryButton.set_sensitive(False) 
        self.categoryArr.append([self.index, self.dummyCategoryButton])
       
        for month in range(1,len(self.data.allMonthMenu) + 1):
            self.dummyContentLabel = Gtk.Label()
            self.dummyContentLabel.set_property("height-request", 30)
            if month == len(self.data.allMonthMenu):
                self.dummyContentLabel.override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.totalColor, self.totalColor, self.totalColor, self.totalColor))
            self.contentArr.append(self.dummyContentLabel)
            self.contentGrid.attach(self.dummyContentLabel, month - 1, self.index, 1, 1) 
        self.entryRows.append([self.index, self.contentArr])
        
        self.categoryGrid.attach(self.dummyCategoryButton, 0, self.index, 1, 1) 
        self.index += 1
    
    def month_clicked(self, button, index):
        self.monthIndex = index - 1
        for i in range (0, len(self.monthArr)):
            if self.monthArr[i][0] == index:
                self.monthArr[i][1].set_relief(Gtk.ReliefStyle.HALF)
                for j in range(0, len(self.entryRows)):
                    if self.entryRows[j][0] == self.categoryIndex:
                        self.entryRows[j][1][i].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.menuColor, self.menuColor, self.menuColor, self.menuColor))
                    else:
                        self.entryRows[j][1][i].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.highlightColor, self.highlightColor, self.highlightColor, self.highlightColor))
            else:
                self.monthArr[i][1].set_relief(Gtk.ReliefStyle.NONE)
                for j in range(0, len(self.entryRows)):
                    if self.entryRows[j][0] != self.categoryIndex:
                       self.entryRows[j][1][i].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(.0, .0, .0, .0));
                    else:
                        self.entryRows[j][1][i].override_background_color(Gtk.StateFlags.NORMAL, Gdk.RGBA(self.highlightColor, self.highlightColor, self.highlightColor, self.highlightColor))
    
    def redisplay_info(self):
        self.index = 0
        while len(self.categoryArr) > 0:
            self.categoryArr.pop(0)
            self.categoryGrid.remove_row(0)
            self.contentGrid.remove_row(0)
        while len(self.entryRows) > 0:
            self.entryRows.pop(0)
        self.display_info("income", self.data.transactions)
        self.empty_row()
        self.display_info("expense", self.data.transactions)
        self.overviewGrid.show_all()
        
        self.clear_selection(self.clearButton)
        
        #self.balanceTotalLabel.set_text( "$" + str(self.calc.sumTotalData(self.data.transactions) - self.calc.sumTotalData(self.data.transactions)))
        #self.varianceTotalLabel.set_text( "$" + str(self.calc.sumTotalData(self.data.transactions)))
        #self.incomeTotalValueLabel.set_text( "$" + str(self.calc.sumTotalData(self.data.transactions)))
        #self.expensesTotalValueLabel.set_text( "$" + str(self.calc.sumTotalData(self.data.transactions)))



from gi.repository import Gtk, Gio, Gdk

class Edit_Entry(Gtk.Window):

    def __init__(self, data, view):
        self.data = data
        self.view = view

        # Content Grid
        self.LAYOUT_GRID_INDEX = 0           # Element

        # Layout Widgets
        self.LAYOUT_WIDGET_INDEX = 1         # Array
        self.CATEGORY_LABEL_INDEX = 0        # Element
        self.DATE_LABEL_INDEX = 1            # Element
        self.CURRENCY_LABEL_INDEX = 2        # Element
        self.COST_LABEL_INDEX = 3            # Element
        self.DESCRIPTION_LABEL_INDEX = 4     # Element
        self.EDIT_BUTTON_INDEX = 5           # Element
        self.TITLE_LABEL_INDEX = 6           # Element
        
        # Additional Items
        self.ENTRY_GRID_INDEX = 2            # Element
        self.COST_GRID_INDEX = 3             # Element
        self.UNIQUE_ID_INDEX = 4             # Element
        
        #Initialize Data
        self.unique_id = 0
        self.entryRow = 0
        self.editPopover = ""
        self.contentGrid = 0
        self.dataEntry = 0

        # Create Widgets
        self.editGrid = Gtk.Grid()
        
        self.editButton = Gtk.Button("Edit")
        self.deleteButton = Gtk.Button("Delete")
        self.selectorBox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        
        self.confirmLabelLine1 = Gtk.Label("Are you sure?")
        self.confirmLabelLine2 = Gtk.Label()
        self.deleteCancelButton = Gtk.Button("Cancel")
        self.deleteConfirmButton = Gtk.Button("Confirm")
        self.deleteSelectorBox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        
        # Style Widgets
        self.editButton.set_size_request(100,32)
        self.deleteButton.set_size_request(100,32)
        self.deleteCancelButton.set_size_request(100,32)
        self.deleteConfirmButton.set_size_request(100,32)
        self.confirmLabelLine2.set_markup("<span foreground=\"red\"><b>This cannot be undone!</b></span>")
        Gtk.StyleContext.add_class(self.selectorBox.get_style_context(), "linked")
        Gtk.StyleContext.add_class(self.deleteSelectorBox.get_style_context(), "linked")
        self.selectorBox.set_margin_start(10)
        self.selectorBox.set_margin_top(10)
        self.selectorBox.set_margin_bottom(10)
        self.selectorBox.set_margin_end(5)
        self.confirmLabelLine1.set_margin_top(10)
        self.deleteSelectorBox.set_margin_start(10)
        self.deleteSelectorBox.set_margin_top(10)
        self.deleteSelectorBox.set_margin_bottom(10)
        self.deleteSelectorBox.set_margin_end(5)

        # Connect Widget Handlers
        self.editButton.connect("clicked", self.on_editButton_clicked)
        self.deleteButton.connect("clicked", self.on_deleteButton_clicked)
        self.deleteCancelButton.connect("clicked", self.on_deleteCancelButton_clicked)
        self.deleteConfirmButton.connect("clicked", self.on_deleteConfirmButton_clicked)

        # Add Widgets to Grid
        self.selectorBox.add(self.editButton)
        self.selectorBox.add(self.deleteButton)

        self.deleteSelectorBox.add(self.deleteCancelButton)
        self.deleteSelectorBox.add(self.deleteConfirmButton)
        
        self.editGrid.attach(self.selectorBox,0,0,2,1)
        self.editGrid.attach(self.confirmLabelLine1, 0, 1, 2, 1)
        self.editGrid.attach(self.confirmLabelLine2, 0, 2, 2, 1)
        self.editGrid.attach(self.deleteSelectorBox, 0, 3, 2, 1)
        
    def margin(self, widget, margin):
        widget.set_margin_start(margin)
        widget.set_margin_top(margin)
        widget.set_margin_bottom(margin)
    
    def on_calendarDropdown_clicked(self, button, calendarPopover):
        if calendarPopover.get_visible():
            calendarPopover.hide()
        else:
            calendarPopover.show_all()
    
    def on_calendarDropdown_closed(self, calendarPopover):
        dateString = self.data.translate_date(self.calendar.get_date(),"edit")
        self.calendarButton.set_label(dateString)

    def on_cancelButton_clicked(self, button):
        self.editGrid.hide()
        self.entryRow[self.LAYOUT_WIDGET_INDEX][self.EDIT_BUTTON_INDEX].show_all()
        self.entryRow[self.ENTRY_GRID_INDEX].show_all()
        self.contentGrid.queue_draw()
    
    def on_deleteButton_clicked(self, *args):
        self.selectorBox.hide()
        self.confirmLabelLine1.show()
        self.confirmLabelLine2.show()
        self.deleteSelectorBox.show_all()
    
    def on_deleteCancelButton_clicked(self, *args):
        self.confirmLabelLine1.hide()
        self.confirmLabelLine2.hide()
        self.deleteSelectorBox.hide()
        self.selectorBox.show()
    
    def on_deleteConfirmButton_clicked(self, *args):
        self.editPopover.hide()
        if self.view == "transaction":
            self.data.delete_transaction(self.unique_id)
        if self.view == "projection":
            self.data.delete_projection(self.unique_id)
        self.contentGrid.queue_draw()
    
    def on_editButton_clicked(self, *args):
        self.editPopover.hide()
        
        # Create editing widgets
        self.editGrid = Gtk.Grid()
        
        if self.view == "projection":
            self.titleLabel = Gtk.Label("Title")
            self.titleEntry = Gtk.Entry()

        self.categoryLabel = Gtk.Label("Category:")
        self.calendarLabel = Gtk.Label("Date:")
        self.costLabel = Gtk.Label("Cost:")
        self.descriptionLabel = Gtk.Label("Description:")
        
        self.categoryComboBoxText = Gtk.ComboBoxText()
        self.costEntry = Gtk.Entry()
        self.descriptionEntry = Gtk.Entry()
        self.submitButton = Gtk.Button("Submit")
        self.cancelButton = Gtk.Button("Cancel")

        self.calendar = Gtk.Calendar()
        self.calendarButton = Gtk.Button()
        self.calendarPopover = Gtk.Popover.new(self.calendarButton)
        self.calendarPopover.add(self.calendar)
        
        # Connect Widgets
        self.calendarButton.connect("clicked", self.on_calendarDropdown_clicked, self.calendarPopover)
        self.calendarPopover.connect("closed", self.on_calendarDropdown_closed)
        self.cancelButton.connect("clicked", self.on_cancelButton_clicked)
        self.submitButton.connect("clicked", self.on_submitButton_clicked)

        # Style Editing Widgets
        if self.view == "projection":
            self.titleLabel.set_halign(Gtk.Align.END)
            self.titleLabel.set_margin_top(20)
            
            self.titleEntry.set_margin_start(32)
            self.titleEntry.set_margin_top(20)        
            
            self.categoryLabel.set_halign(Gtk.Align.END)
            self.categoryLabel.set_margin_top(8)
        
            self.categoryComboBoxText.set_margin_start(32)
            self.categoryComboBoxText.set_margin_top(8)
        
        if self.view == "transaction":
            self.categoryLabel.set_halign(Gtk.Align.END)
            self.categoryLabel.set_margin_top(20)
            
            self.categoryComboBoxText.set_margin_start(32)
            self.categoryComboBoxText.set_margin_top(20)
        
        self.calendarLabel.set_halign(Gtk.Align.END)
        self.calendarLabel.set_margin_top(8)

        self.calendarButton.set_margin_start(32)
        self.calendarButton.set_margin_top(8)
        
        self.costLabel.set_halign(Gtk.Align.END)
        self.costLabel.set_margin_top(8)

        self.costEntry.set_alignment(1)
        self.costEntry.set_margin_start(32)
        self.costEntry.set_margin_top(8)
        
        self.descriptionLabel.set_halign(Gtk.Align.END)
        self.descriptionLabel.set_margin_top(8)

        self.descriptionEntry.set_margin_start(32)
        self.descriptionEntry.set_margin_top(8)
        
        self.cancelButton.set_alignment(1, 1)
        self.cancelButton.set_margin_top(32)
        self.cancelButton.set_margin_bottom(32)

        self.submitButton.set_margin_start(32)
        self.submitButton.set_margin_top(32)
        self.submitButton.set_margin_bottom(20)

        self.cancelButton.set_margin_top(32)
        self.cancelButton.set_margin_bottom(20)
       
        # Set Calendar to currently set date
        self.set_calendar()
 
        # Replace label widgets with editing widgets
        #if self.view == "transaction":
        for j in range(0,len(self.data.transactionsMenu)):
            if self.data.transactionsMenu[j][self.data.MENU_NAME_INDEX] != "Uncategorized":
                self.categoryComboBoxText.append_text(self.data.transactionsMenu[j][self.data.MENU_NAME_INDEX])
        if self.entryRow[self.LAYOUT_WIDGET_INDEX][self.CATEGORY_LABEL_INDEX].get_text() != "Uncategorized":
            for j in range(0,len(self.data.transactionsMenu)-2):
                self.categoryComboBoxText.set_active(j)
                if self.categoryComboBoxText.get_active_text() == self.entryRow[self.LAYOUT_WIDGET_INDEX][self.CATEGORY_LABEL_INDEX].get_text():
                    break
        
        # Style Edit Grid, Hide Entry Grid
        self.editGrid.set_halign(Gtk.Align.CENTER)
        self.editGrid.set_hexpand(True)
        self.entryRow[self.LAYOUT_WIDGET_INDEX][self.EDIT_BUTTON_INDEX].hide()
        self.entryRow[self.ENTRY_GRID_INDEX].hide()
       
        # Title
        if self.view == "projection":
            self.titleEntry.set_text(self.entryRow[self.LAYOUT_WIDGET_INDEX][self.TITLE_LABEL_INDEX].get_text())
            self.editGrid.attach(self.titleLabel, 1,0,1,1)
            self.editGrid.attach(self.titleEntry,2,0,1,1)
        
        # Category
        self.editGrid.attach(self.categoryLabel, 1,1,1,1)
        self.editGrid.attach(self.categoryComboBoxText,2,1,1,1)
        
        # Date
        self.calendarButton.set_label(self.entryRow[self.LAYOUT_WIDGET_INDEX][self.DATE_LABEL_INDEX].get_text())
        self.editGrid.attach(self.calendarLabel,1,2,1,1)
        self.editGrid.attach(self.calendarButton,2,2,1,1)
        
        # Cost
        self.costEntry.set_text(self.entryRow[self.LAYOUT_WIDGET_INDEX][self.COST_LABEL_INDEX].get_text())
        self.editGrid.attach(self.costLabel,1,3,1,1)
        self.editGrid.attach(self.costEntry,2,3,1,1)
        
        # Description
        self.descriptionEntry.set_text(self.entryRow[self.LAYOUT_WIDGET_INDEX][self.DESCRIPTION_LABEL_INDEX].get_text())
        self.editGrid.attach(self.descriptionLabel,1,4,1,1)
        self.editGrid.attach(self.descriptionEntry,2,4,1,1)
        
        # Add Submit/Cancel Button to popover
        self.editGrid.attach(self.cancelButton,1,5,1,1)
        self.editGrid.attach(self.submitButton,2,5,1,1)

        # Attach and Show Edit Grid
        self.entryRow[self.LAYOUT_GRID_INDEX].attach(self.editGrid, 0, 0, 1, 1)
        self.contentGrid.queue_draw()
        self.editGrid.show_all()
    
    def on_editDropdown_clicked(self, button, editPopover, unique_id, entryRow, contentGrid, dataEntry):
        if editPopover.get_visible():
            editPopover.hide()
        else:
            editPopover.show_all()
            self.confirmLabelLine1.hide()
            self.confirmLabelLine2.hide()
            self.deleteSelectorBox.hide()

        self.editPopover = editPopover
        self.unique_id = unique_id
        self.entryRow = entryRow
        self.contentGrid = contentGrid
        self.dataEntry = dataEntry

    def on_submitButton_clicked(self, button):
        if self.categoryComboBoxText.get_active() < 0:
            self.categoryLabel.set_markup("<span foreground=\"red\"><b>* Category</b></span>")
        else:
            self.editGrid.hide()
            self.dateArr = self.calendar.get_date()
            self.year = str(self.dateArr[0])
            self.month = str(self.dateArr[1] + 1)
            self.day = str(self.dateArr[2])

            if self.view == "transaction":
                self.data.update_transaction(self.categoryComboBoxText.get_active_text(), self.year, self.month, self.day, 
                                    self.costEntry.get_text(), self.descriptionEntry.get_text(), self.unique_id)
            elif self.view == "projection":
                self.data.update_projection(self.titleEntry.get_text(), self.categoryComboBoxText.get_active_text(), self.year, self.month, self.day, 
                                    self.costEntry.get_text(), self.descriptionEntry.get_text(), self.unique_id)
 
            self.entryRow[self.LAYOUT_WIDGET_INDEX][self.EDIT_BUTTON_INDEX].show_all()
            self.entryRow[self.ENTRY_GRID_INDEX].show_all()
            
            self.contentGrid.queue_draw()
    
    def set_calendar(self):
        if self.view == "transaction":
            self.calendar.select_month(self.dataEntry[self.data.TRANSACTION_DATE_INDEX][self.data.TRANSACTION_DATE_MONTH_INDEX] - 1,
                self.dataEntry[self.data.TRANSACTION_DATE_INDEX][self.data.TRANSACTION_DATE_YEAR_INDEX])
            self.calendar.select_day(self.dataEntry[self.data.TRANSACTION_DATE_INDEX][self.data.TRANSACTION_DATE_DAY_INDEX])
        
        if self.view == "projection":
            self.calendar.select_month(self.dataEntry[self.data.PROJECTIONS_START_MONTH] - 1,
                                        self.dataEntry[self.data.PROJECTIONS_START_YEAR])
            self.calendar.select_day(self.dataEntry[self.data.PROJECTIONS_START_DAY])



from gi.repository import Gtk, Gio, Gdk
from budget.overview import Overview
from budget.transactions import Transactions
from budget.projections import Projections
from budget.add_popover import Add_Entry_Popover

class Window(Gtk.Window):

    def __init__(self, data):
        self.module_dir = (__file__)
        while (len(self.module_dir)-1 > 0 and self.module_dir[len(self.module_dir)-1] != "/"):
            self.module_dir = self.module_dir.replace(" ", "").rstrip(self.module_dir[-1:])
        
        self.data = data 
        self.provider = Gtk.CssProvider()
        self.provider.load_from_path(self.module_dir + "style.css")
        Gtk.StyleContext.add_provider_for_screen(Gdk.Screen.get_default(), self.provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        Gtk.Window.__init__(self, title="Budget")
        self.set_default_size(1300, 850)
        
        # Initialize Views
        self.overview = Overview(self.data)
        self.transactions = Transactions(self.data)
        self.projections = Projections(self.data)

        # --- Header Bar ---
        # Create Header Bar
        self.headerBox = Gtk.Box(name="headerBox")
        self.hb = Gtk.HeaderBar(name="hb")
       
        # Style Header Bar
        self.hb.set_show_close_button(True)
        self.hb.set_hexpand(True)

        self.headerBox.add(self.hb)
        self.set_titlebar(self.headerBox)

        # --- Navigation Buttons ---
        # Create Navigation Buttons
        self.overviewRadio = Gtk.RadioButton.new_with_label(None, "Overview")
        self.transactionsRadio = Gtk.RadioButton.new_with_label(None, "Transactions")
        self.projectionsRadio = Gtk.RadioButton.new_with_label(None, "Projections")
        
        self.transactionsRadio.join_group(self.overviewRadio)
        self.projectionsRadio.join_group(self.overviewRadio)
        
        self.radioNavBox = Gtk.Box(Gtk.Orientation.HORIZONTAL,1)
        #self.radioNavBox.pack_start(self.overviewRadio, True, True, 0)
        self.radioNavBox.pack_start(self.transactionsRadio, True, True, 0)
        self.radioNavBox.pack_start(self.projectionsRadio, True, True, 0)
        
        # Style Navigation Buttons
        self.radioNavBox.set_homogeneous(True)
        self.radioNavBox.set_property("height-request", 32)
        #self.overviewRadio.set_property("draw-indicator",False)
        self.transactionsRadio.set_property("draw-indicator",False)
        self.projectionsRadio.set_property("draw-indicator",False)
        Gtk.StyleContext.add_class(self.radioNavBox.get_style_context(), "linked")

        # Connect Radio Buttons to handler
        #self.overviewRadio.connect("toggled", self.on_navRadio_toggled)
        self.transactionsRadio.connect("toggled", self.on_navRadio_toggled)
        self.projectionsRadio.connect("toggled", self.on_navRadio_toggled)
        
        self.radioStatus = "transactions"

        # Add Navigation Radio Buttons to Navigation Box
        self.hb.set_custom_title(self.radioNavBox)
        
        # --- Action Buttons ---
        # Create Widgets
        self.addButton = Gtk.Button()
        self.menuButton = Gtk.MenuButton();
        
        self.addEntryPopover = Gtk.Popover.new(self.addButton)
        self.add_entry_popover = Add_Entry_Popover(self.data)
        self.addEntryPopover.add(self.add_entry_popover.addGrid)
       
        # Add Images
        self.addIcon = Gio.ThemedIcon(name="list-add-symbolic")
        self.menuIcon = Gio.ThemedIcon(name="open-menu-symbolic")
        self.addImage = Gtk.Image.new_from_gicon(self.addIcon, Gtk.IconSize.MENU)
        self.menuImage = Gtk.Image.new_from_gicon(self.menuIcon, Gtk.IconSize.MENU)
        self.addButton.add(self.addImage)
        self.menuButton.add(self.menuImage)
       
        # Style Action Buttons
        self.addButton.set_size_request(32,32)
        self.menuButton.set_size_request(32,32)
        
        # Connect to handler
        self.addButton.connect("clicked", self.add_entry_popover.on_addButton_clicked, self.addEntryPopover)
        self.menuButton.connect("clicked", self.on_menuButton_clicked)
        
        # Pack Header Bar 
        self.hb.pack_end(self.menuButton)
        self.hb.pack_end(self.addButton)
        
        # --- Notebooks ---
        # Create Labels
        #self.overviewLabel = Gtk.Label("Overview")
        self.transactionsLabel = Gtk.Label("Transactions")
        self.projectionsLabel = Gtk.Label("Projections")

        # Style Notebook 
        #self.overviewLabel.set_hexpand(True)
        self.transactionsLabel.set_hexpand(True)
        self.projectionsLabel.set_hexpand(True)
       
        # Create Notebook
        self.notebook = Gtk.Notebook()
        #self.notebook.insert_page(self.overview.grid, self.overviewLabel, 0)
        self.notebook.insert_page(self.transactions.grid, self.transactionsLabel, 1)
        self.notebook.insert_page(self.projections.grid, self.projectionsLabel, 2)
        self.add(self.notebook)
        self.notebook.set_show_tabs(False)
        
        # Connect to handler
        # self.notebook.connect("switch-page", self.on_notebook_switch)
      
        # Connect views to data
        self.data.connect_data_views(self.transactions, self.overview, self.projections)
    
    def on_addButton_clicked(self, *args):
        if self.addEntryPopover.get_visible():
            self.addEntryPopover.hide()
        else:
            self.addEntryPopover.show_all()
    
    def on_menuButton_clicked(self, *args):
        print("Menu Button Working!")
    
    def on_navRadio_toggled(self, *args):
        if self.overviewRadio.get_active() == True:
            self.radioStatus = "overview"
            self.switch_to_overview()
        elif self.transactionsRadio.get_active() == True:
            self.radioStatus = "transactions"
            self.switch_to_transactions()
        elif self.projectionsRadio.get_active() == True:
            self.radioStatus = "projections"
            self.switch_to_projections()

    def switch_to_overview(self):
        self.notebook.set_current_page(0)
     
    def switch_to_transactions(self):
        self.notebook.set_current_page(0)
    
    def switch_to_projections(self):
        self.notebook.set_current_page(1)



from gi.repository import Gtk, Gio
import datetime, calendar
from budget.edit_mode import Edit_Entry

class Projections():
        
    def __init__(self, data):
        self.data = data
        self.projections = []
        self.defaultLoadOut = 180
        self.grid = Gtk.Grid(name="projectionsGrid")        
        self.entryRows = []
        self.totalSum = 0  
        
        self.currentDay = datetime.datetime.now().day
        self.currentMonth = datetime.datetime.now().month
        self.currentYear = datetime.datetime.now().year

        # Create Grids
        self.sideGrid = Gtk.Grid()
        self.contentGrid = Gtk.Grid()
        self.headerGrid = Gtk.Grid(name="headerGrid")
        self.dayViewGrid = Gtk.Grid()
        self.weekViewGrid = Gtk.Grid()
        self.monthViewGrid = Gtk.Grid()
        self.yearViewGrid = Gtk.Grid()
        self.transactionViewGrid = Gtk.Grid( name="transactionViewGrid")
        
        self.gridScrolledWindow = Gtk.ScrolledWindow()
        self.gridViewport = Gtk.Viewport(name="transactionViewport")
        
        # Side Grid Styling 
        self.sideGrid.set_vexpand(True)
        self.sideGrid.set_property("width-request",279)
        
        # Content Grid Styling 
        self.contentGrid.set_hexpand(True)
        self.contentGrid.set_vexpand(True)
        self.contentGrid.set_column_homogeneous(True)
    
        # --- Notebooks ---
        # Style Grids
        self.monthViewGrid.set_column_homogeneous(True)
        self.monthViewGrid.set_margin_start(55)
        self.monthViewGrid.set_margin_end(55)
        self.monthViewGrid.set_margin_bottom(55)

        self.transactionViewGrid.set_hexpand(True)
        self.transactionViewGrid.set_halign(Gtk.Align.FILL)
        self.transactionViewGrid.set_margin_start(55)
        self.transactionViewGrid.set_margin_top(20)
        self.transactionViewGrid.set_margin_end(55)
        self.transactionViewGrid.set_margin_bottom(55)

        # Create Labels
        self.dayLabel = Gtk.Label("Day")
        self.weekLabel = Gtk.Label("Week")
        self.monthLabel = Gtk.Label("Month")
        self.yearLabel = Gtk.Label("Year")
        self.transactionLabel = Gtk.Label("Transactions")

        # Create Notebooks
        self.projectionsNotebook = Gtk.Notebook()
        #self.projectionsNotebook.insert_page(self.dayViewGrid, self.dayLabel,0)
        #self.projectionsNotebook.insert_page(self.weekViewGrid, self.weekLabel,1)
        self.projectionsNotebook.insert_page(self.monthViewGrid, self.monthLabel,2)
        #self.projectionsNotebook.insert_page(self.yearViewGrid, self.yearLabel,3)
        self.projectionsNotebook.set_show_tabs(False)
        
        self.gridScrolledWindow.add(self.gridViewport)
        self.gridViewport.add(self.contentGrid)
 
        self.generate_sidebar()
        self.generate_transactions_view()
        self.generate_month_view()
        self.create_header_grid()

        self.contentGrid.attach(self.headerGrid, 1, 0, 1, 1)
        self.contentGrid.attach(self.projectionsNotebook, 1, 1, 1, 1)
        self.contentGrid.attach(self.transactionViewGrid, 1, 2, 1, 1)
        #self.grid.attach(self.sideGrid,0,0,1,1)
        self.grid.attach(self.gridScrolledWindow,1,0,1,1)

    def active_day_cell(self, dayText, incomeSum, expenseSum, totalSum, incomeLabel, expenseLabel, totalLabel):
        if incomeSum != 0:
            incomeLabel.set_markup("<span foreground=\"green\">" + "+" + str("%0.2f" % (incomeSum,)) + "</span>")
        if expenseSum != 0:
            expenseLabel.set_markup("<span foreground=\"red\">" + "-" + str("%0.2f" % (expenseSum,)) + "</span>")
        if incomeSum != 0 or expenseSum != 0:
            self.totalSum += incomeSum 
            self.totalSum -= expenseSum
            if self.totalSum < 0:
                totalLabel.set_markup("<span foreground=\"red\">" + "=" + str("%0.2f" % (self.totalSum,)) + "</span>")
            if self.totalSum >= 0:
                totalLabel.set_markup("<span foreground=\"green\">" + "=" + str("%0.2f" % (self.totalSum,)) + "</span>")
    
    def add_projection(self, i, startYear, startMonth, startDay):
        self.arr = []
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_TITLE])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_VALUE])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_DESCRIPTION])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_CATEGORY_ID])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_CATEGORY_NAME])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_CATEGORY_TYPE])
        self.arr.append(startYear)
        self.arr.append(startMonth)
        self.arr.append(startDay)
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_END_YEAR])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_END_MONTH])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_END_DAY])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_FREQUENCY])
        self.arr.append(self.data.projections[i][self.data.PROJECTIONS_ID])
        
        self.projections = self.data.sort_projections(self.projections, self.arr)

    def create_header_grid(self):
        self.viewLabel = Gtk.Label("View:")
        self.viewCombo = Gtk.ComboBoxText()
        self.addButton = Gtk.Button("Add Projection")
        
        # Generate Months
        self.viewCombo.append_text("Day")
        self.viewCombo.append_text("Week")
        self.viewCombo.append_text("Month")
        self.viewCombo.append_text("Year")
        
        # Connect to handler    
        self.viewCombo.connect("changed",self.view_selected)
        
        self.viewCombo.set_active(0)

        self.viewLabel.set_margin_start(30)
        self.viewLabel.set_margin_end(5)
        
        self.viewCombo.set_margin_top(10)
        self.viewCombo.set_margin_bottom(10)
        self.viewCombo.set_margin_end(10)
        
        self.addButton.set_margin_start(20)
        self.addButton.set_margin_top(10)
        self.addButton.set_margin_bottom(10)
   
        self.headerGrid.attach(self.viewLabel,0,0,1,1)
        self.headerGrid.attach(self.viewCombo,1,0,1,1)
        self.headerGrid.attach(self.addButton,2,0,1,1)
       
        # Create Widgets
        self.addPopover = Gtk.Popover.new(self.addButton)
        self.addPopoverGrid = Gtk.Grid()
        self.addPopover.add(self.addPopoverGrid)
        self.addButton.connect("clicked", self.on_addButton_clicked)


        self.transactionTitleLabel = Gtk.Label("Title:")
        self.transactionTitleEntry = Gtk.Entry()
        
        self.transactionAmountLabel = Gtk.Label("Amount:")
        self.transactionAmountEntry = Gtk.Entry()
        
        self.transactionDescriptionLabel = Gtk.Label("Description:")
        self.transactionDescriptionEntry = Gtk.Entry()
        
        self.transactionTypeLabel = Gtk.Label("Type:")
        self.addIncomeRadio = Gtk.RadioButton.new_with_label(None, "Income")
        self.addExpenseRadio = Gtk.RadioButton.new_with_label(None, "Expense")
        self.addExpenseRadio.join_group(self.addIncomeRadio)
        self.radioBox = Gtk.Box(Gtk.Orientation.HORIZONTAL,1)
        self.radioBox.pack_start(self.addIncomeRadio, True, True, 0)
        self.radioBox.pack_start(self.addExpenseRadio, True, True, 0)
        
        self.addCategoryLabel = Gtk.Label("Category:")
        self.addCategoryComboBoxText = Gtk.ComboBoxText()
        
        self.frequencyLabel = Gtk.Label("Frequency:")
        self.frequencyComboBoxText = Gtk.ComboBoxText()
            
        self.startDateLabel = Gtk.Label("Date:")
        self.startDate = Gtk.Calendar()
        
        self.neverRadio = Gtk.RadioButton.new_with_label(None, "Never")
        self.selectDateRadio = Gtk.RadioButton.new_with_label(None, "Select Date")
        self.selectDateRadio.join_group(self.neverRadio)

        self.endDateLabel = Gtk.Label("End Date:")
        self.endDate = Gtk.Calendar()

        self.cancelButton = Gtk.Button("Cancel")
        self.submitButton = Gtk.Button("Submit")
            
        # Add data to frequency
        for j in range(0, len(self.data.frequencyMenu)):
            self.frequencyComboBoxText.append_text(self.data.frequencyMenu[j][0])        

        self.transactionAmountEntry.connect("insert-text", self.data.check_amount_value)
        self.frequencyComboBoxText.connect("changed",self.frequency_selected)
        self.selectDateRadio.connect("toggled", self.on_selectDateRadio_toggled)
        
        # Style Widgets
        self.transactionTitleLabel.set_halign(Gtk.Align.END)
        self.transactionTitleLabel.set_margin_start(20)
        self.transactionTitleLabel.set_margin_top(10)
        self.transactionTitleEntry.set_margin_start(20)
        self.transactionTitleEntry.set_margin_top(10)
        self.transactionTitleEntry.set_margin_end(20)
        
        self.transactionAmountLabel.set_halign(Gtk.Align.END)
        self.transactionAmountLabel.set_margin_start(20)
        self.transactionAmountLabel.set_margin_top(10)
        self.transactionAmountEntry.set_margin_start(20)
        self.transactionAmountEntry.set_margin_top(10)
        self.transactionAmountEntry.set_margin_end(20)
        
        self.transactionDescriptionLabel.set_halign(Gtk.Align.END)
        self.transactionDescriptionLabel.set_margin_start(20)
        self.transactionDescriptionLabel.set_margin_top(10)
        self.transactionDescriptionEntry.set_margin_start(20)
        self.transactionDescriptionEntry.set_margin_top(10)
        self.transactionDescriptionEntry.set_margin_end(20)
        
        self.transactionTypeLabel.set_halign(Gtk.Align.END)
        self.transactionTypeLabel.set_margin_start(20)
        self.transactionTypeLabel.set_margin_top(10)
        self.radioBox.set_margin_start(20)
        self.radioBox.set_margin_top(10)
        self.radioBox.set_margin_end(20)
        self.radioBox.set_property("height-request", 34)
        self.addIncomeRadio.set_property("draw-indicator",False)
        self.addExpenseRadio.set_property("draw-indicator",False)
        Gtk.StyleContext.add_class(self.radioBox.get_style_context(), "linked")

        self.addCategoryLabel.set_halign(Gtk.Align.END)
        self.addCategoryLabel.set_margin_start(20)
        self.addCategoryLabel.set_margin_top(10)
        self.addCategoryComboBoxText.set_margin_start(20)
        self.addCategoryComboBoxText.set_margin_top(10)
        self.addCategoryComboBoxText.set_margin_end(20)
        self.addCategoryComboBoxText.set_property("height-request", 34)
        
        self.frequencyLabel.set_halign(Gtk.Align.END)
        self.frequencyLabel.set_margin_start(20)
        self.frequencyLabel.set_margin_top(10)
        self.frequencyComboBoxText.set_margin_start(20)
        self.frequencyComboBoxText.set_margin_top(10)
        self.frequencyComboBoxText.set_margin_end(20)
        self.frequencyComboBoxText.set_property("height-request", 34)
        
        self.startDateLabel.set_halign(Gtk.Align.END)
        self.startDateLabel.set_margin_start(20)
        self.startDateLabel.set_margin_top(10)
        self.startDate.set_margin_start(20)
        self.startDate.set_margin_top(10)
        self.startDate.set_margin_end(20)
        
        self.endDateLabel.set_halign(Gtk.Align.END)
        self.endDateLabel.set_margin_start(20)
        self.endDateLabel.set_margin_top(10)
        
        self.selectDateRadio.set_margin_start(20)
        self.selectDateRadio.set_margin_top(10)
        
        self.neverRadio.set_margin_start(20)
        self.neverRadio.set_margin_top(10)
        self.neverRadio.set_margin_end(20)
        
        self.endDate.set_margin_start(20)
        self.endDate.set_margin_top(10)
        self.endDate.set_margin_end(20)
        
        self.cancelButton.set_margin_start(20)
        self.cancelButton.set_margin_top(30)
        self.cancelButton.set_margin_bottom(20)
        self.submitButton.set_margin_start(20)
        self.submitButton.set_margin_top(30)
        self.submitButton.set_margin_bottom(20)
        self.submitButton.set_margin_end(20)
        
        # Connect Widgets
        self.cancelButton.connect("clicked", self.on_cancelButton_clicked)
        self.submitButton.connect("clicked", self.on_submitButton_clicked)
        self.addIncomeRadio.connect("toggled", self.on_addRadio_toggled)
        
        # Attach Widgets
        if len(self.projections) == 0:
            self.transactionViewGrid.set_halign(Gtk.Align.CENTER)
        else:
            self.transactionViewGrid.set_halign(Gtk.Align.FILL)

        self.addPopoverGrid.attach(self.transactionTitleLabel,1,1,1,1)
        self.addPopoverGrid.attach(self.transactionTitleEntry,2,1,2,1)
        self.addPopoverGrid.attach(self.transactionAmountLabel,1,2,1,1)
        self.addPopoverGrid.attach(self.transactionAmountEntry,2,2,2,1)
        self.addPopoverGrid.attach(self.transactionDescriptionLabel,1,3,1,1)
        self.addPopoverGrid.attach(self.transactionDescriptionEntry,2,3,2,1)
        self.addPopoverGrid.attach(self.transactionTypeLabel,1,4,1,1)
        self.addPopoverGrid.attach(self.radioBox,2,4,2,1)
        self.addPopoverGrid.attach(self.addCategoryLabel,1,5,1,1)
        self.addPopoverGrid.attach(self.addCategoryComboBoxText,2,5,2,1)
        self.addPopoverGrid.attach(self.frequencyLabel,1,6,1,1)
        self.addPopoverGrid.attach(self.frequencyComboBoxText,2,6,2,1)
        self.addPopoverGrid.attach(self.startDateLabel,4,1,1,4)
        self.addPopoverGrid.attach(self.startDate,5,1,2,4)
        self.addPopoverGrid.attach(self.endDateLabel,4,5,1,1)
        self.addPopoverGrid.attach(self.neverRadio,5,5,1,1)
        self.addPopoverGrid.attach(self.selectDateRadio,6,5,1,1)
        self.addPopoverGrid.attach(self.endDate,5,6,2,4)
        self.addPopoverGrid.attach(self.cancelButton,4,10,1,1)
        self.addPopoverGrid.attach(self.submitButton,5,10,2,1)        
    
    def frequency_selected(self, listbox, *args):
        # To catch calls before widget exists.
        if listbox == None:
            return
        else:
            self.row = self.data.frequencyMenu[listbox.get_active()][0]
            if listbox.get_active() == 0:
                self.startDateLabel.set_text("Date:")
                self.endDateLabel.hide()
                self.selectDateRadio.hide()
                self.neverRadio.hide()
                self.endDate.hide()
                self.transactionViewGrid.queue_draw()
            else:
                self.startDateLabel.set_text("Start Date:")
                self.selectDateRadio.show()
                self.neverRadio.show()
                self.endDateLabel.show()
                self.transactionViewGrid.queue_draw()

    def generate_transactions_view(self):
        # Create new array full of projections
        for i in range(0,len(self.data.projections)):
            startYear = self.data.projections[i][self.data.PROJECTIONS_START_YEAR]
            startMonth = self.data.projections[i][self.data.PROJECTIONS_START_MONTH]
            startDay = self.data.projections[i][self.data.PROJECTIONS_START_DAY]
            currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
            
            start = datetime.date(self.data.projections[i][self.data.PROJECTIONS_START_YEAR],
                    self.data.projections[i][self.data.PROJECTIONS_START_MONTH],
                    self.data.projections[i][self.data.PROJECTIONS_START_DAY])
            
            if self.data.projections[i][self.data.PROJECTIONS_END_YEAR] != 0:
                end = datetime.date(self.data.projections[i][self.data.PROJECTIONS_END_YEAR],
                        self.data.projections[i][self.data.PROJECTIONS_END_MONTH],
                        self.data.projections[i][self.data.PROJECTIONS_END_DAY])
                days = (end-start).days
                if days > self.defaultLoadOut:
                    days = self.defaultLoadOut
            else:
                days = self.defaultLoadOut
            
            # Create Daily Projections
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "One Time":
                self.add_projection(i, startYear, startMonth, startDay)
           
            # Create Daily Projections
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Daily":
                for j in range(days):
                    self.add_projection(i, startYear, startMonth, startDay)
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1
            
            # Create Weekly Projections
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Weekly":
                dayOfWeek = datetime.datetime(startYear, startMonth, startDay).weekday()
                for j in range(days):
                    if dayOfWeek == datetime.datetime(startYear, startMonth, startDay).weekday():
                        self.add_projection(i, startYear, startMonth, startDay)
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1
            
            # Create Bi-Weekly Projections
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Bi-Weekly":
                dayOfWeek = datetime.datetime(startYear, startMonth, startDay).weekday()
                self.flag = True;
                for j in range(days):
                    if dayOfWeek == datetime.datetime(startYear, startMonth, startDay).weekday():
                        if self.flag == True:
                            self.add_projection(i, startYear, startMonth, startDay)
                            self.flag = False
                        else:
                            self.flag = True
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1
            
            # Create Monthly On Date Projections
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Monthly on Date":
                recurringDate = startDay
                for j in range(days):
                    if startDay == recurringDate:
                        self.add_projection(i, startYear, startMonth, startDay)
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1
                        
            # Create Monthly on Weekday
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Monthly on Weekday":
                dayOfWeek = datetime.datetime(startYear, startMonth, startDay).weekday()
                self.count = 0;
                for j in range(days):
                    if dayOfWeek == datetime.datetime(startYear, startMonth, startDay).weekday():
                        if self.count == 0:
                            self.add_projection(i, startYear, startMonth, startDay)
                            self.count += 1
                        elif self.count != 3:
                            self.count += 1
                        elif self.count == 3:
                            self.count = 0;
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1
            
            # Create Monthly on Weekday
            if self.data.projections[i][self.data.PROJECTIONS_FREQUENCY] == "Yearly":
                dayOfWeek = datetime.datetime(startYear, startMonth, startDay).weekday()
                self.recurringYear = startYear
                self.recurringMonth = startMonth
                self.recurringDay = startDay
                for j in range(days):
                    if self.recurringMonth == startMonth and self.recurringDay == startDay:
                            self.add_projection(i, startYear, startMonth, startDay)
                    if startDay >= currentMonthEndDate:
                        startDay = 1
                        if startMonth == 12:
                            startMonth = 1
                            startYear += 1
                        else:
                            startMonth += 1
                        currentMonthEndDate = calendar.monthrange(startYear, startMonth)[1] 
                    startDay += 1

        # Code from transactions.py
        while len(self.transactionViewGrid) > 0:
            self.transactionViewGrid.remove_row(0)
            
        while len(self.entryRows) > 0:
                self.entryRows.pop(0)
        self.index = 10
        self.count = 0
        for i in reversed(range(0,len(self.projections))):
            if self.projections[i][6] >= self.currentYear:
                if self.projections[i][7] >= self.currentMonth:
                    if self.projections[i][8] >= self.currentDay:
                        # Date String
                        self.dateString = [self.projections[i][self.data.PROJECTIONS_START_YEAR],self.projections[i][self.data.PROJECTIONS_START_MONTH] - 1,self.projections[i][self.data.PROJECTIONS_START_DAY]]
                        self.dateString = self.data.translate_date(self.dateString, "edit")
                        
                        self.layoutGrid = Gtk.Grid(name="layoutGrid")
                        self.entryGrid = Gtk.Grid()
                        self.costGrid = Gtk.Grid()
                 
                        self.titleLabel = Gtk.Label()
                        self.categoryLabel = Gtk.Label()
                        self.dateLabel = Gtk.Label(self.dateString)
                        self.descriptionLabel = Gtk.Label()
                        
                        self.currencyLabel = Gtk.Label("$")
                        self.costLabel = Gtk.Label()
                        
                        self.costLabel.set_markup("<span foreground=\"green\">" + str(self.projections[i][self.data.PROJECTIONS_VALUE]) + "</span>")
                       
                        self.titleLabel.set_markup("<b>" + self.projections[i][self.data.PROJECTIONS_TITLE] + "</b>")
                        self.descriptionLabel.set_markup("<i>" + self.projections[i][self.data.PROJECTIONS_DESCRIPTION] + "</i>")
                        self.categoryLabel.set_markup(self.projections[i][self.data.PROJECTIONS_CATEGORY_NAME])
                        
                        # Create Edit Popover
                        self.editButton = Gtk.Button()
                        self.editView = Gtk.Popover.new(self.editButton)
                        self.edit_view = Edit_Entry(self.data, "projection")
                        self.editView.add(self.edit_view.editGrid)
                        
                        # Style Widgets
                        self.entryGrid.set_halign(Gtk.Align.CENTER)
                        self.entryGrid.set_hexpand(True)
                        self.categoryLabel.set_property("height-request", 50)
                        self.categoryLabel.set_property("xalign", 1)
                        self.categoryLabel.set_width_chars(15)
                        
                        self.dateLabel.set_margin_start(30)
                        self.dateLabel.set_margin_end(30)
                        self.dateLabel.set_width_chars(15)
                        
                        self.costGrid.set_row_homogeneous(True)
                        self.costLabel.set_property("xalign", .05)
                        self.costLabel.set_width_chars(14)
                        
                        # Style Edit Button
                        self.editIcon = Gio.ThemedIcon(name="go-down-symbolic")
                        self.editImage = Gtk.Image.new_from_gicon(self.editIcon, Gtk.IconSize.MENU)
                        self.editButton.add(self.editImage)
                        self.editButton.set_relief(Gtk.ReliefStyle.NONE)
                        self.editButton.set_valign(Gtk.Align.START)
                        self.editButton.set_opacity(.5)

                        # Attach Labels
                        self.costGrid.attach(self.currencyLabel, 0,1,1,1)
                        self.costGrid.attach(self.costLabel, 1,1,1,1)
                        
                        self.entryGrid.attach(self.titleLabel, 0, 0, 3, 1)
                        self.entryGrid.attach(self.categoryLabel, 0, 2, 1, 1)
                        self.entryGrid.attach(self.dateLabel, 1, 2, 1, 1)
                        self.entryGrid.attach(self.costGrid, 2, 1, 1, 2)

                        if self.descriptionLabel.get_text() != "":
                            self.entryGrid.attach(self.descriptionLabel, 0, 3, 3, 1)
                            self.extraSpaceLabel = Gtk.Label()
                            self.entryGrid.attach(self.extraSpaceLabel,0, 4, 1, 1)
                            self.layoutGrid.attach(self.entryGrid, 0, 0, 1, 5)
                        
                        # Add Layout Grid to Content Grid. Increment index and apply whitespaces
                        else:
                            self.layoutGrid.attach(self.entryGrid, 0, 0, 1, 2)
                        
                        self.layoutGrid.attach(self.editButton, 1, 0, 1, 1)
                        self.layoutGrid.set_margin_bottom(25)


                        self.transactionViewGrid.attach(self.layoutGrid, 1, self.index, 4, 2)

                        self.index = self.index + 2

                        self.transactionType = ""
                        
                        self.entryRows.append([self.layoutGrid, [self.categoryLabel, self.dateLabel, self.currencyLabel, self.costLabel, self.descriptionLabel, self.editButton, self.titleLabel], self.entryGrid, self.costGrid, self.projections[i][self.data.PROJECTIONS_ID]])
                        self.editButton.connect("clicked", self.edit_view.on_editDropdown_clicked, self.editView, self.projections[i][self.data.PROJECTIONS_ID], self.entryRows[self.count],  self.transactionViewGrid, self.projections[i])
                        self.count += 1
                        self.transactionViewGrid.show_all() 
 
    def generate_month_view(self):
        self.monthTitleLabel = Gtk.Label()
        self.monthPreviousButton = Gtk.Button()
        self.monthNextButton = Gtk.Button()
        
        self.backIcon = Gio.ThemedIcon(name="go-previous-symbolic")
        self.backImage = Gtk.Image.new_from_gicon(self.backIcon, Gtk.IconSize.MENU)
        self.monthPreviousButton.add(self.backImage)
        
        self.forwardIcon = Gio.ThemedIcon(name="go-next-symbolic")
        self.forwardImage = Gtk.Image.new_from_gicon(self.forwardIcon, Gtk.IconSize.MENU)
        self.monthNextButton.add(self.forwardImage)
        
        self.monthPreviousButton.connect("clicked", self.on_monthPreviousButton_clicked)
        self.monthNextButton.connect("clicked", self.on_monthNextButton_clicked)

        self.monthPreviousButton.set_margin_top(10)
        self.monthPreviousButton.set_relief(Gtk.ReliefStyle.NONE)
        
        self.monthNextButton.set_margin_top(10)
        self.monthNextButton.set_relief(Gtk.ReliefStyle.NONE)

        self.monthTitleLabel.set_margin_top(10)
        self.monthTitleLabel.set_halign(Gtk.Align.CENTER)
        self.monthTitleLabel.set_hexpand(True)

        self.monthCalendarGrid = Gtk.Grid()        

        self.monthViewGrid.attach(self.monthPreviousButton, 2,0,1,1)
        self.monthViewGrid.attach(self.monthTitleLabel, 1,0,5,1)
        self.monthViewGrid.attach(self.monthNextButton, 4,0,1,1)
        self.monthViewGrid.attach(self.monthCalendarGrid, 0,1,7,1)
         
        self.selectedMonth = self.currentMonth
        self.selectedYear = self.currentYear

        self.currentMonthStartDate = calendar.monthrange(datetime.datetime.now().year, datetime.datetime.now().month)[0]
        self.currentMonthEndDate = calendar.monthrange(datetime.datetime.now().year, datetime.datetime.now().month)[1] 

        self.populate_month_grid(self.currentMonth, self.currentYear, self.currentMonthStartDate, self.currentMonthEndDate)

    def generate_sidebar(self):
        return
 
    def inactive_day_cell(self, dayText, incomeSum, expenseSum, totalSum, incomeLabel, expenseLabel, totalLabel):
        if incomeSum != 0:
            incomeLabel.set_markup("<span foreground=\"gray\">" + "+" + str("%0.2f" % (incomeSum,)) + "</span>")
        if expenseSum != 0:
            expenseLabel.set_markup("<span foreground=\"gray\">" + "-" + str("%0.2f" % (expenseSum,)) + "</span>")

    def on_addButton_clicked(self, *args):
        if self.addPopover.get_visible():
            self.addPopover.hide()
        else:
            self.reset_fields()
            self.addPopover.show_all()
            self.neverRadio.hide()
            self.selectDateRadio.hide()
            self.endDateLabel.hide()
            self.endDate.hide()
        
    def on_submitButton_clicked(self, *args):
        if self.transactionTitleEntry.get_text() == "":
            self.transactionTitleLabel.set_markup("<span foreground=\"red\"><b>* Title</b></span>")
        else:
            self.transactionTitleLabel.set_text("Title")

        if self.transactionAmountEntry.get_text() == "":
            self.transactionAmountLabel.set_markup("<span foreground=\"red\"><b>* Amount</b></span>")
        else:
            self.transactionAmountLabel.set_text("Amount")
        
        if self.addCategoryComboBoxText.get_active() < 0:
            self.addCategoryLabel.set_markup("<span foreground=\"red\"><b>* Category</b></span>")
        else:
            self.addCategoryLabel.set_text("Category")
        
        if self.frequencyComboBoxText.get_active() < 0:
            self.frequencyLabel.set_markup("<span foreground=\"red\"><b>* Frequency</b></span>")
        else:
            self.frequencyLabel.set_text("Frequency")
        
        if (self.transactionTitleEntry.get_text() != "" 
            and self.transactionAmountEntry.get_text() != "" 
            and self.addCategoryComboBoxText.get_active() >= 0 
            and self.frequencyComboBoxText.get_active() >= 0):
            
            self.dateArr = self.startDate.get_date()
            self.startYear = str(self.dateArr[0])
            self.startMonth = str(self.dateArr[1] + 1)
            self.startDay = str(self.dateArr[2])
            
            if self.frequencyComboBoxText.get_active() == 0:
                self.dateArr = self.startDate.get_date()
                self.endYear = str(self.dateArr[0])
                self.endMonth = str(self.dateArr[1] + 1)
                self.endDay = str(self.dateArr[2])
            else:
                if self.selectDateRadio.get_active() == True:
                    self.dateArr = self.endDate.get_date()
                    self.endYear = str(self.dateArr[0])
                    self.endMonth = str(self.dateArr[1] + 1)
                    self.endDay = str(self.dateArr[2])
                elif self.neverRadio.get_active() == True:
                    self.endYear = str(0)
                    self.endMonth = str(0)
                    self.endDay = str(0)
            
            if self.addIncomeRadio.get_active() == True:
                self.selected = "income"
            elif self.addExpenseRadio.get_active() == True:
                self.selected = "expense"

            self.data.LATEST_PROJECTION_ID += 1
            
            self.data.add_projection(self.transactionTitleEntry.get_text(),
                self.transactionAmountEntry.get_text(), self.transactionDescriptionEntry.get_text(),
                self.selected, self.addCategoryComboBoxText.get_active(), self.startYear, self.startMonth, self.startDay, 
                self.endYear, self.endMonth, self.endDay, self.frequencyComboBoxText.get_active(), self.data.LATEST_PROJECTION_ID)
            
            self.reset_fields()
            self.addPopover.hide()

    def on_addRadio_toggled(self, *args):
        for i in range(0, len(self.data.transactionsMenu)):
            self.addCategoryComboBoxText.remove(0)
        if self.addIncomeRadio.get_active() == True:
            self.radioStatus = "income"
            for i in range(0,len(self.data.transactionsMenu)):
                if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == "income":
                    if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                        self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
        elif self.addExpenseRadio.get_active() == True:
            self.radioStatus = "expense"
            for i in range(0,len(self.data.transactionsMenu)):
                if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == "expense":
                    if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                        self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
    
    def on_cancelButton_clicked(self, *args):
        if self.addPopover.get_visible():
            self.addPopover.hide()
        else:
            self.addPopover.show_all()

    def on_selectDateRadio_toggled(self, *args):
        if self.selectDateRadio.get_active() == True:
            self.endDate.show()
            self.frequencyLabel.set_margin_bottom(103)
            self.frequencyComboBoxText.set_margin_bottom(103)
            self.transactionViewGrid.queue_draw()
        if self.neverRadio.get_active() == True:
            self.endDate.hide()
            self.frequencyLabel.set_margin_bottom(0)
            self.frequencyComboBoxText.set_margin_bottom(0)
            self.transactionViewGrid.queue_draw()
    
    def on_monthPreviousButton_clicked(self, *args):
        if self.selectedMonth == 1:
            self.selectedYear = self.selectedYear - 1
            self.selectedMonth = 12
        else:
            self.selectedMonth = self.selectedMonth - 1
        
        start = calendar.monthrange(self.selectedYear, self.selectedMonth)[0]
        end = calendar.monthrange(self.selectedYear, self.selectedMonth)[1] 
        self.populate_month_grid(self.selectedMonth, self.selectedYear, start, end)

    def on_monthNextButton_clicked(self, *args):
        if self.selectedMonth == 12:
            self.selectedYear = self.selectedYear + 1
            self.selectedMonth = 1
        else:
            self.selectedMonth = self.selectedMonth + 1
        start = calendar.monthrange(self.selectedYear, self.selectedMonth)[0]
        end = calendar.monthrange(self.selectedYear, self.selectedMonth)[1] 
        self.populate_month_grid(self.selectedMonth, self.selectedYear, start, end)
    
    def populate_month_grid(self, month, year, start, end):
        if month == 1:
            self.monthTitleLabel.set_text("January")
        elif month == 2:
            self.monthTitleLabel.set_text("February")
        elif month == 3:
            self.monthTitleLabel.set_text("March")
        elif month == 4:
            self.monthTitleLabel.set_text("April")
        elif month == 5:
            self.monthTitleLabel.set_text("May")
        elif month == 6:
            self.monthTitleLabel.set_text("June")
        elif month == 7:
            self.monthTitleLabel.set_text("July")
        elif month == 8:
            self.monthTitleLabel.set_text("August")
        elif month == 9:
            self.monthTitleLabel.set_text("September")
        elif month == 10:
            self.monthTitleLabel.set_text("October")
        elif month == 11:
            self.monthTitleLabel.set_text("November")
        elif month == 12:
            self.monthTitleLabel.set_text("December")

        self.monthTitleLabel.set_text(self.monthTitleLabel.get_text() + " " + str(self.selectedYear))
        
        if self.monthCalendarGrid.get_child_at(1,1) != None:
            for i in range(1,8):
                self.monthCalendarGrid.get_child_at(i,1).destroy()
                self.monthCalendarGrid.get_child_at(i,2).destroy()
                self.monthCalendarGrid.get_child_at(i,3).destroy()
                self.monthCalendarGrid.get_child_at(i,4).destroy()
                self.monthCalendarGrid.get_child_at(i,5).destroy()
                self.monthCalendarGrid.get_child_at(i,6).destroy()
        
        self.dayText = 1
        self.nextMonthDate = 1

        for i in range(0, len(self.data.transactions)):
            for j in range(0, len(self.data.incomeMenu)):
                if self.data.transactions[i][self.data.TRANSACTION_MENU_INDEX][self.data.TRANSACTION_MENU_ID_INDEX] == self.data.incomeMenu[j]:
                    self.totalSum += self.data.transactions[i][self.data.TRANSACTION_VALUE_INDEX]
       
        for i in range(0, len(self.data.transactions)):
            for j in range(0, len(self.data.expenseMenu)):
                if self.data.transactions[i][self.data.TRANSACTION_MENU_INDEX][self.data.TRANSACTION_MENU_ID_INDEX] == self.data.expenseMenu[j]:
                    self.totalSum -= self.data.transactions[i][self.data.TRANSACTION_VALUE_INDEX]

        for i in range(1, 43):
            # Create Widgets 
            if i - 2 >= start and self.dayText <= end:
                if self.selectedYear == self.currentYear and self.selectedMonth == self.currentMonth and int(self.dayText) == self.currentDay:
                    if i == 7 or i == 14 or i == 21 or i == 28 or i == 35:
                        self.monthGrid = Gtk.Grid(name="monthGridRightCurrent")
                    elif i > 35 and i < 42:
                        self.monthGrid = Gtk.Grid(name="monthGridBottomCurrent")
                    elif i == 42:
                        self.monthGrid = Gtk.Grid(name="monthGridLastCurrent")
                    else:            
                        self.monthGrid = Gtk.Grid(name="monthGridCurrent")
                else:
                    if i == 7 or i == 14 or i == 21 or i == 28 or i == 35:
                        self.monthGrid = Gtk.Grid(name="monthGridRightActive")
                    elif i > 35 and i < 42:
                        self.monthGrid = Gtk.Grid(name="monthGridBottomActive")
                    elif i == 42:
                        self.monthGrid = Gtk.Grid(name="monthGridLastActive")
                    else:
                        self.monthGrid = Gtk.Grid(name="monthGridActive")
                
                self.dayLabel = Gtk.Label(self.dayText)
                self.dayText += 1

                self.expenseLabel = Gtk.Label()
                self.incomeLabel = Gtk.Label()
                self.totalLabel = Gtk.Label()
                
                self.incomeSum = 0
                self.expenseSum = 0

                for j in range(0,len(self.projections)):
                    if self.projections[j][self.data.PROJECTIONS_START_YEAR] == self.selectedYear:
                        if self.projections[j][self.data.PROJECTIONS_START_MONTH] == self.selectedMonth:
                            if self.projections[j][self.data.PROJECTIONS_START_DAY] == self.dayText - 1:
                                if self.projections[j][self.data.PROJECTIONS_CATEGORY_TYPE] == 0:
                                    self.incomeSum += self.projections[j][self.data.PROJECTIONS_VALUE]
                                if self.projections[j][self.data.PROJECTIONS_CATEGORY_TYPE] == 1:
                                    self.expenseSum += self.projections[j][self.data.PROJECTIONS_VALUE]
                
                if self.currentYear == self.selectedYear:
                    if self.currentMonth == self.selectedMonth:
                        if int(self.dayText) >= self.currentDay:
                            self.active_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)
                        elif int(self.dayText) < self.currentDay:
                            self.inactive_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)
                    if self.currentMonth < self.selectedMonth:                
                        self.active_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)
                    elif self.currentMonth > self.selectedMonth:                
                        self.inactive_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)
                elif self.currentYear < self.selectedYear: 
                    self.active_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)
                elif self.currentYear > self.selectedYear:
                    self.inactive_day_cell(self.dayText, self.incomeSum, self.expenseSum, self.totalSum, self.incomeLabel, self.expenseLabel, self.totalLabel)

                self.monthGrid.attach(self.dayLabel,0,0,1,1)
                self.monthGrid.attach(self.incomeLabel,0,1,1,1)
                self.monthGrid.attach(self.expenseLabel,0,2,1,1)            
                self.monthGrid.attach(self.totalLabel,0,3,1,1)

            else:
                if i == 7 or i == 14 or i == 21 or i == 28 or i == 35:
                    self.monthGrid = Gtk.Grid(name="monthGridRightInactive")
                elif i > 35 and i < 42:
                    self.monthGrid = Gtk.Grid(name="monthGridBottomInactive")
                elif i == 42:
                    self.monthGrid = Gtk.Grid(name="monthGridLastInactive")
                else:            
                    self.monthGrid = Gtk.Grid(name="monthGridInactive")
                
                if i - 2 < start: 

                    if month != 1:
                        self.prevMonthEndDate = calendar.monthrange(year, month - 1)[1]
                        self.prevMonthDate = self.prevMonthEndDate - start + i - 1
                        self.dayLabel = Gtk.Label(self.prevMonthDate)
                        self.monthGrid.attach(self.dayLabel,0,0,1,1)
                    else:
                        self.prevMonthEndDate = calendar.monthrange(year, 12)[1]
                        self.prevMonthDate = self.prevMonthEndDate - start + i - 1
                        self.dayLabel = Gtk.Label(self.prevMonthDate)
                        self.monthGrid.attach(self.dayLabel,0,0,1,1)
                        
                elif self.dayText > end:
                    self.dayLabel = Gtk.Label(self.nextMonthDate)
                    self.monthGrid.attach(self.dayLabel,0,0,1,1)
                    self.nextMonthDate += 1

            # Style Widgets
            self.monthGrid.set_column_homogeneous(True)
            self.dayLabel.set_margin_end(10)
            self.dayLabel.set_margin_top(5)
            self.dayLabel.set_halign(Gtk.Align.END)
            self.monthCalendarGrid.set_column_homogeneous(True)
            self.monthCalendarGrid.set_row_homogeneous(True)
            #self.monthCalendarGrid.set_vexpand(True)
            self.monthCalendarGrid.set_margin_top(20)
        
            if i <= 7:
                self.monthCalendarGrid.attach(self.monthGrid,i,1,1,1)
            elif i > 7 and i <= 14:
                self.monthCalendarGrid.attach(self.monthGrid,(i-7),2,1,1)
            elif i > 14 and i <= 21:
                self.monthCalendarGrid.attach(self.monthGrid,(i-14),3,1,1)
            elif i > 21 and i <= 28:
                self.monthCalendarGrid.attach(self.monthGrid,(i-21),4,1,1)
            elif i > 28 and i <= 35:
                self.monthCalendarGrid.attach(self.monthGrid,(i-28),5,1,1)
            elif i > 35:
                self.monthCalendarGrid.attach(self.monthGrid,(i-35),6,1,1)

            self.monthCalendarGrid.show_all()
    
    def redisplay_info(self):
        self.projections = []
        self.generate_transactions_view()
        self.populate_month_grid(self.currentMonth, self.currentYear, self.currentMonthStartDate, self.currentMonthEndDate)
        self.transactionViewGrid.queue_draw()
        
    def reset_fields(self):
        for i in range(0, len(self.data.transactionsMenu)):
            self.addCategoryComboBoxText.remove(0)

        if self.addIncomeRadio.get_active() == True:
            self.selected = "income"
        elif self.addExpenseRadio.get_active() == True:
            self.selected = "expense"
        for i in range(0,len(self.data.transactionsMenu)):
            if self.data.transactionsMenu[i][self.data.MENU_TYPE_INDEX] == self.selected:
                if self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX] != "Uncategorized":
                    self.addCategoryComboBoxText.append_text(self.data.transactionsMenu[i][self.data.MENU_NAME_INDEX])
        
        # Reset fields to default
        self.transactionTitleEntry.set_text("")
        self.transactionAmountEntry.set_text("")
        self.transactionDescriptionEntry.set_text("")
        self.addIncomeRadio.set_active(True)
        self.addCategoryComboBoxText.set_active(-1)
        self.frequencyComboBoxText.set_active(-1)
        self.startDateLabel.set_text("Date:")
        self.startDate.select_month(self.currentMonth - 1, self.currentYear)
        self.startDate.select_day(self.currentDay)
        self.neverRadio.set_active(True)
        self.endDate.select_month(self.currentMonth - 1, self.currentYear)
        self.endDate.select_day(self.currentDay)
        
    
    def view_selected(self, listbox, *args):
        self.projectionsNotebook.set_current_page(listbox.get_active())
        
        



from setuptools import setup, find_packages

try:
    long_description = open("README.rst").read()
except IOError:
    long_description = ""

ENTRY_POINTS = {
    'console_scripts': ['budget=budget.budget:main'],
    #gui_scripts=['app_gui=budget.buget:start']
}


setup(
    name="budget",
    version="0.0.4",
    description="Personal Finance",
    license="GPL v2.0",
    author="Marc Thomas",
    author_email = "mat@mthx.org",
    url = "http://github.com/mthxx/budget",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    long_description=long_description,
    entry_points=ENTRY_POINTS,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
    ]
)
